/***************************************************************************
                          automountpluginfactory.cpp  -  description
                             -------------------
    begin                : Mon Nov 29 1999
    copyright            : (C) 1999 by Henrik Stormer
    email                : stormer@ifi.unizh.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "automountpluginfactory.h"
#include "automountplugin.h"
#include "sys/mount.h"
#include "knc.h"
#include "mountoptiontab.h"
#include "mountdialog.h"
#include "qmessagebox.h"
#define PRIORITY 5


AutomountPluginFactory::AutomountPluginFactory() : BasePluginFactory(PRIORITY)
{
  KConfig* Config = kapp->getConfig();

  fstab* fs;
  while ((fs = getfsent()) != NULL)
  {
    if (QString(fs->fs_mntops).find("noauto") != -1)
    {
      FStab* fstab = new FStab(fs);
      bool isAuto = Config->readBoolEntry(QString("Automount: ") + fs->fs_file, true);
      fstab->setAutomount(isAuto);
      _MountData.insert(fstab->getFile().data(), fstab);
    }
  }
  connect (&_P, SIGNAL(receivedStderr (KProcess*, char*, int)),
           this, SLOT (receivedStderr (KProcess*, char*, int)));

  connect (&_P, SIGNAL(receivedStdout (KProcess*, char*, int)),
           this, SLOT (receivedStdout (KProcess*, char*, int)));

}


AutomountPluginFactory::~AutomountPluginFactory()
{
   KConfig* Config=kapp->getConfig();
  QDictIterator<FStab> it (_MountData);
  Config->setGroup("Automount");
  for (;it.current();++it)
    Config->writeEntry(QString("Automount: ") + it.current()->getFile(), it.current()->getAutomount());
}

void AutomountPluginFactory::receivedStderr(KProcess* proc, char* buffer, int length)
{
  QString Msg (buffer, length+1);
  QMessageBox::information(NULL,i18n("Error"),Msg, QMessageBox::Ok|QMessageBox::Default);
}


void AutomountPluginFactory::receivedStdout(KProcess* proc, char* buffer, int length)
{
  QString Msg (buffer, length+1);
  QMessageBox::information(NULL,i18n("Error"),Msg, QMessageBox::Ok|QMessageBox::Default);
}

bool AutomountPluginFactory::isResponsible(QString Path, QString DownPath)
{
  FStab* fs;
  if ((fs = _MountData[Path+DownPath]) != NULL)
  {
    if (fs->getAutomount())
    {
      _P.clearArguments();
      _P << "mount | grep" << fs->getFile()  << "> /dev/null || mount"  <<  fs->getFile();
      _P.start(KProcess::Block, KProcess::Stderr);
    }
  }
  return false;
}

bool AutomountPluginFactory::addMenuItems(QPopupMenu* menu)
{
 menu->insertItem(i18n("&Mount Device"), ID_AUTOMOUNT_MOUNT);
 menu->insertItem(i18n("&Unmount Device"), ID_AUTOMOUNT_UNMOUNT);
 return true;
}

bool AutomountPluginFactory::addToolbarItems(KToolBar*)
{
 return false;
}

bool AutomountPluginFactory::commandCallback(int id)
{
 switch (id)
 {
  case ID_AUTOMOUNT_MOUNT: slotMount(); return true;
  case ID_AUTOMOUNT_UNMOUNT: slotUnmount(); return true;
  default: return false;
 }
}

bool AutomountPluginFactory::statusCallback(int id, QString& StatusMsg)
{
 switch (id)
 {
  case ID_AUTOMOUNT_MOUNT: StatusMsg = i18n("Shows a dialog for mounting a device"); return true;
  case ID_AUTOMOUNT_UNMOUNT: StatusMsg = i18n("Shows a dialog for unmounting a device"); return true;
  default: return false;
 }
}

void AutomountPluginFactory::slotMount()
{
  MountDialog Dialog(i18n("Mount"), _MountData);
  if (Dialog.exec())
  {
    _P.clearArguments();
    _P << "mount" << "|" << "grep" << Dialog.getType() << "> /dev/null" << "||" << "mount"  << Dialog.getType();
    _P.start(KProcess::Block, KProcess::AllOutput);
  }
}

void AutomountPluginFactory::slotUnmount()
{
  MountDialog Dialog(i18n("Unmount"), _MountData);
  if (Dialog.exec())
  {
      _P.clearArguments();
      _P << "mount" << "|" << "grep" << Dialog.getType() << "> /dev/null" << "&&" << "umount"  << Dialog.getType();
      _P.start(KProcess::Block, KProcess::AllOutput);
  }
}

BaseOptionTab* AutomountPluginFactory::getOptionsTab()
{
  return new MountOptionTab(_MountData);
}

void AutomountPluginFactory::initKeyAccel (KAccel* accel, KNCApp* App)
{
  accel->insertItem (i18n("Mount Device"), "Mount Device", CTRL+Key_M);
  accel->insertItem (i18n("Unmount Device"), "Unmount Device", CTRL+Key_U);
}

void AutomountPluginFactory::changeMenuAccel (KAccel* accel, QPopupMenu* menu)
{
  accel->disconnectItem ("Mount Device", this, SLOT (slotMount() ) );
  accel->connectItem ("Mount Device", this, SLOT (slotMount() ) );

  accel->disconnectItem ("Unmount Device", this, SLOT (slotUnmount() ) );
  accel->connectItem ("Unmount Device", this, SLOT (slotUnmount() ) );

  accel->changeMenuAccel(menu, ID_AUTOMOUNT_MOUNT ,"Mount Device" );
  accel->changeMenuAccel(menu, ID_AUTOMOUNT_UNMOUNT ,"Unmount Device" );
}