/***************************************************************************
                          automountpluginfactory.h  -  description                              
                             -------------------                                         
    begin                : Mon Nov 29 1999                                           
    copyright            : (C) 1999 by Henrik Stormer                         
    email                : stormer@ifi.unizh.ch                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef AUTOMOUNTPLUGINFACTORY_H
#define AUTOMOUNTPLUGINFACTORY_H

#include <qobject.h>
#include <automountplugin.h>
/**
  *@author Henrik Stormer
  */

#include <baseplugin.h>
#include <fstab.h>
#include <basepluginfactory.h>
#include <kprocess.h>
/**
  *@author Henrik Stormer
  */

#define ID_AUTOMOUNT_MOUNT 5120
#define ID_AUTOMOUNT_UNMOUNT 5125

class KConfig;

class FStab
{
  public:
  FStab(fstab* fs) : _Spec(fs->fs_spec), _File(fs->fs_file), _VFSType(fs->fs_vfstype), _MntOps(fs->fs_mntops),
                     _Type(fs->fs_type), _Freq(fs->fs_freq), _Passno(fs->fs_passno), _Automount(true) {}

  QString getSpec() {return _Spec;}
  QString getFile() {return _File;}
  QString getVFSType() {return _VFSType;}
  QString getMntOps() {return _MntOps;}
  QString getType() {return _Type;}
  int getFreq() {return _Freq;}
  int getPassno() {return _Passno;}
  bool getAutomount() {return _Automount;}
  void setAutomount(bool Setting) {_Automount = Setting;}
 private:
  QString _Spec, _File, _VFSType, _MntOps, _Type;
  int     _Freq, _Passno;
  bool    _Automount;

};

class MountOptionTab;

class AutomountPluginFactory : public BasePluginFactory
{
 Q_OBJECT
 public:
  AutomountPluginFactory();
  ~AutomountPluginFactory();
  virtual bool isResponsible(QString Path, QString SubPath);

  virtual QString getType() {return "AutomountPlugin";}
  virtual BasePlugin* generatePlugin(QString Dir, QString SubDir) {return NULL;}

  virtual BaseOptionTab* getOptionsTab();
  virtual QString  getOptionsTabName() {return i18n("Automount");}

  virtual bool addMenuItems(QPopupMenu*);
  virtual bool addToolbarItems(KToolBar*);
  virtual bool commandCallback(int id);
  virtual bool statusCallback(int id, QString& StatusMsg);
  virtual void initKeyAccel (KAccel*, KNCApp*);
  virtual void changeMenuAccel (KAccel*, QPopupMenu*);
 public slots:
   void slotMount();
   void slotUnmount();
   void receivedStdout(KProcess* proc, char* buffer, int length);
   void receivedStderr(KProcess* proc, char* buffer, int length);
 protected:
   void slotShowMountDialog();
   void slotShowUnmountDialog();

  private:
    QDict<FStab>  _MountData;
    KShellProcess _P;
};


#endif
