/***************************************************************************
                          basefileinfo.h  -  description                              
                             -------------------                                         
    begin                : Tue Oct 5 1999                                           
    copyright            : (C) 1999 by Henrik Stormer                         
    email                : stormer@ifi.unizh.ch                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef BASEFILEINFO_H
#define BASEFILEINFO_H

#include <qstring.h>
#include <qdatetime.h>
#include <progressdialog.h>
#include <deletedialog.h>
#include <kapp.h>
#include <kiconloader.h>
/**
  *@author Henrik Stormer
  */

class BaseFileInfo
{
 public:
	BaseFileInfo();
	virtual ~BaseFileInfo();
  virtual QString fileName() const {return _Name;}
  virtual QString filePath() const {return _Path + _Name;}
  virtual QString dirPath() const {return _Path;}
  virtual double  size() const {return _dSize;}
  virtual QString getSize() const {QString Help; Help.setNum(_dSize, 'g', 25);return Help;}
  virtual QString getAttrib() const {return _Attrib;}
  virtual long    getAttribute() const = 0;
  virtual void    setAttribute(long Attribute) = 0;

  virtual QString getOwner() {return _Owner;}
  virtual QString getGroup() {return _Group;}

  virtual void setOwner(QString Owner) {}
  virtual void setGroup(QString Group) {}

  virtual QDateTime lastModified() const {return _LastModified;}
  virtual QDateTime lastRead() const {return _LastRead;}

  virtual bool isFile() const {return true;}
  virtual bool isDir() const {return false;}
  virtual bool isLink() const {return false;}
  virtual QString infoAsString () = 0;
  virtual BaseFileInfo* clone () = 0;

  virtual bool accessRead() const = 0;
  virtual bool accessWrite() const = 0;
  virtual bool accessExecute() const = 0;

  virtual bool copy(ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& stop) {return false;}
  virtual bool rename(ProgressDialog* PFile, QString Target, bool& bOverwrite, bool& stop) {return false;}
  virtual bool remove(DeleteDialog* PFile, bool& stop) {return false;}

  virtual QByteArray data() {return QByteArray();}
  virtual QPixmap getIcon() {return kapp->getIconLoader()->loadMiniIcon("unknown.xpm");}
  virtual QString getFile() {return filePath();}
 protected:
  QString   _Name;
  QString   _Path;
  double    _dSize;
  QString   _Attrib;
  QString   _Owner;
  QString   _Group;
  QDateTime _LastModified;
  QDateTime _LastRead;
};

/*
class BaseDirInfo : public BaseDirInfo
{
 public:
   BaseDirInfo();
   virtual ~BaseDirInfo();

   virtual bool isFile() const {return false;}
   virtual bool isDir() const {return true;}

   double  getCountFiles() const {return _countFiles;}
   QString getCountFilesAsString() const { QString Help; return Help.setNum(_countFiles, 'g', 25);}


   virtual void    setDirEntry(double countfiles, double NewSize);
   virtual bool    isSizeSet() {return _bSizeSet;}


  virtual  BaseDirInfo* clone () = 0;
 protected:
   bool    _bSizeSet;
   double  _countFiles;
};
*/

#endif











