/***************************************************************************
                          basepluginfactory.h  -  description                              
                             -------------------                                         
    begin                : Mon Nov 29 1999                                           
    copyright            : (C) 1999 by Henrik Stormer                         
    email                : stormer@ifi.unizh.ch                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef BASEPLUGINFACTORY_H
#define BASEPLUGINFACTORY_H

#include <qobject.h>
#include <baseplugin.h>

class BaseOptionTab;

/**
  *@author Henrik Stormer
  */
class KNCApp;
class BasePluginFactory : public QObject
{
  Q_OBJECT
  public:
  friend BasePlugin;

  BasePluginFactory(int Priority, KNCApp* App=NULL);
  ~BasePluginFactory();

  KNCApp* getApp() {return _App;}

  virtual int getPriority();

  virtual bool isResponsible(QString Path, QString SubPath) = 0;
  virtual BasePlugin* generatePlugin(QString Dir, QString SubDir) = 0;

  //some integration possibilities
  //first the Options dialog
  virtual BaseOptionTab* getOptionsTab();
  virtual QString        getOptionsTabName();

  //then the menu bar for adding new menu items
  virtual bool addMenuItems(QPopupMenu*);

  //and at last the toolbar
  virtual bool addToolbarItems(KToolBar*);

  //if you add Items-> don't forget to overwrite this
  virtual bool commandCallback(int id);
  virtual bool statusCallback(int id, QString& StatusMsg);
  virtual void initKeyAccel (KAccel*, KNCApp*);
  virtual void changeMenuAccel (KAccel*, QPopupMenu*);

  private:
    int     _Priority;
    KNCApp* _App;
};

#endif
