/***************************************************************************
                          ftpconnectiondialog.cpp  -  description
                             -------------------
    begin                : Fri Dec 10 1999
    copyright            : (C) 1999 by Henrik Stormer
    email                : stormer@ifi.unizh.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <qmsgbox.h>
#include "ftpconnectiondialog.h"
#include "selectdialog.h"
#include "ftpextendeddialog.h"
#include <iostream.h>


FtpConnectionDialog::FtpConnectionDialog(QList<FtpConfiguration>& ConnectionList)
  : BaseDialog(), _ConnectionList(ConnectionList), _ActConfiguration(NULL)
{
	initDialog();
  _Password->setEchoMode(QLineEdit::Password);
  QListIterator<FtpConfiguration> it (_ConnectionList);

  if (it.current()) enableDialog(true);

  for (;it.current();++it)
   _Connections->insertItem(it.current()->_Name);
  _ColumnOrder.setAutoDelete(true);

  connect(_Connections, SIGNAL(highlighted(int)), SLOT(slotHighlighted(int)));
  connect(_Anonymous, SIGNAL(toggled(bool)), SLOT(slotAnonymous(bool)));
  _Connections->setSelected(0,true);
  _Connections->setFocus();

  connect(_Connect, SIGNAL(clicked()), SLOT(slotOk()));
  connect(_Cancel, SIGNAL(clicked()), SLOT(slotCancel()));
  connect(_NewEntry, SIGNAL(clicked()), SLOT(slotNewEntry()));
  connect(_DeleteEntry, SIGNAL(clicked()), SLOT(slotDeleteEntry()));
  connect(_SaveEntry, SIGNAL(clicked()), SLOT(slotSaveEntry()));
  connect(_Extended, SIGNAL(clicked()), SLOT(slotExtended()));
}

FtpConnectionDialog::~FtpConnectionDialog(){
}

void FtpConnectionDialog::enableDialog(bool bEnable)
{
  _Host->setEnabled(bEnable);
  _Dir->setEnabled(bEnable);
  _Port->setEnabled(bEnable);
  _Anonymous->setEnabled(bEnable);
}

void FtpConnectionDialog::saveEntries()
{
  if (!_ActConfiguration) return;
  if (QString (_Host->text()) == "")
  {
    QMessageBox::information(NULL, i18n("Error"), i18n("Please insert a proper host name like 'ftp.kde.org'."),QMessageBox::Ok);
    return;
  }
  _ActConfiguration->_Host = _Host->text();
  _ActConfiguration->_Dir = _Dir->text();
  if (_ActConfiguration->_Dir.right(1) != "/") _ActConfiguration->_Dir += "/";

  _ActConfiguration->_Anonymous = _Anonymous->isChecked();
  if (!_ActConfiguration->_Anonymous)
  {
    _ActConfiguration->_User = _User->text();
    _ActConfiguration->_Password = _Password->text();
  }
  _ActConfiguration->_Port = QString(_Port->text()).toInt();
  _ActConfiguration->_ColumnOrder.clear();
  QListIterator<short> it(_ColumnOrder);
  for (;it.current();++it)
    _ActConfiguration->_ColumnOrder.append(new short(*it.current()));

}

void FtpConnectionDialog::slotHighlighted(int Index)
{
  QListIterator<FtpConfiguration> it (_ConnectionList);
  QString Temp = _Connections->text(Index);
  while (it.current()->_Name != _Connections->text(Index)) {++it;}
  _ActConfiguration = it.current();
  _ActConfigurationIndex = Index;
  _Host->setText(it.current()->_Host);
  _Dir->setText(it.current()->_Dir);
  _Anonymous->setChecked(it.current()->_Anonymous);
  if (!_Anonymous->isChecked())
  {
    _User->setText(it.current()->_User);
    _Password->setText(it.current()->_Password);
  }
  _Port->setValue(it.current()->_Port);
  _ColumnOrder.clear();
  QListIterator<short> itc(it.current()->_ColumnOrder);
  for (;itc.current();++itc)
     _ColumnOrder.append(new short(*itc.current()));

}

void FtpConnectionDialog::slotNewEntry()
{
  SelectDialog Dialog(0,0,i18n("Connection Name"));
  if (Dialog.exec())
  {
   _ConnectionList.append(new FtpConfiguration(Dialog.getText()));
   _Connections->insertItem(Dialog.getText());
   _Connections->setSelected(_Connections->count()-1, true);
   if (_ConnectionList.count() == 1) enableDialog(true);
  }
}

void FtpConnectionDialog::slotDeleteEntry()
{
  FtpConfiguration* Config = _ActConfiguration;

  _Connections->removeItem(_ActConfigurationIndex);
  _ConnectionList.remove(Config);
  if (_ConnectionList.count() == 0) enableDialog(false);
}

void FtpConnectionDialog::slotSaveEntry()
{
 saveEntries();
}

void FtpConnectionDialog::slotAnonymous(bool Checked)
{
  _User->setEnabled(!Checked);
  _Password->setEnabled(!Checked);
}

void FtpConnectionDialog::slotExtended()
{
  FtpExtendedDialog Dialog(_ColumnOrder);
  if (Dialog.exec())
    Dialog.getColumnList(_ColumnOrder);
}