/***************************************************************************
                          ftpfileinfo.cpp  -  description                              
                             -------------------                                         
    begin                : Sat Oct 16 1999                                           
    copyright            : (C) 1999 by Henrik Stormer                         
    email                : stormer@ifi.unizh.ch                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include "ftpfileinfo.h"
#include "qdatetime.h"
#include "qstring.h"
#include "qfileinfo.h"
#include "ftpplugin.h"
#include "qimage.h"
#include "qpainter.h"
#include "qbitmap.h"
#include <qmsgbox.h>

FtpFileInfo::FtpFileInfo(FtpPlugin* Plugin, QString Path, QString Name, double Size, QString Attrib, QString Owner, QString Group, QDateTime LastModified, QDateTime LastRead)
  : BaseFileInfo(), _Plugin(Plugin), _Mode(0)
{
  _Path = Path;
  _Name = Name;
  _dSize = Size;
  _Attrib = Attrib;
  _Owner = Owner;
  _Group = Group;

  if (Attrib[0] == 'r') _Mode |= QFileInfo::ReadUser;
  if (Attrib[1] == 'w') _Mode |= QFileInfo::WriteUser;
  if (Attrib[2] == 'x') _Mode |= QFileInfo::ExeUser;

  if (Attrib[3] == 'r') _Mode |= QFileInfo::ReadGroup;
  if (Attrib[4] == 'w') _Mode |= QFileInfo::WriteGroup;
  if (Attrib[5] == 'x') _Mode |= QFileInfo::ExeGroup;

  if (Attrib[6] == 'r') _Mode |= QFileInfo::ReadOther;
  if (Attrib[7] == 'w') _Mode |= QFileInfo::WriteOther;
  if (Attrib[8] == 'x') _Mode |= QFileInfo::ExeOther;

  _LastRead = LastRead;
  _LastModified = LastModified;
}
FtpFileInfo::~FtpFileInfo()
{
// ~BaseFileInfo();
}

FtpDirInfo::FtpDirInfo(FtpPlugin* Plugin, QString Path, QString Name, double Size, QString Attrib, QString Owner, QString Group, QDateTime LastModified, QDateTime LastRead)
 : FtpFileInfo(Plugin, Path, Name, Size, Attrib, Owner, Group, LastModified, LastRead)
{}

FtpDirInfo::~FtpDirInfo()
{
// ~FtpFileInfo();
}


long FtpFileInfo::getAttribute() const
{
  return _Mode;
}
void FtpFileInfo::setAttribute(long Attribute)
{
 _Plugin->setAttribute(this, Attribute);
}
QString FtpFileInfo::infoAsString ()
{
 return _Name;
}
bool FtpFileInfo::accessRead() const
{
 return true;
}
bool FtpFileInfo::accessWrite() const
{
 return true;
}
bool FtpFileInfo::accessExecute() const
{
 return true;
}

bool FtpFileInfo::copy(ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& stop)
{
 return _Plugin->downloadFile(this, PDialog, Target, bOverwrite, stop);
}
bool FtpFileInfo::rename(ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& stop)
{
 return _Plugin->rename(this, PDialog, Target, bOverwrite, stop);
}
bool FtpFileInfo::remove(DeleteDialog* Dialog, bool& stop)
{
 return _Plugin->removeFile(this, Dialog, stop);
}

bool FtpDirInfo::copy(ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& stop)
{
 QString Message;
 up:
  QDir Dir(Target);
  QString Name = Dir.dirName();
  Dir.cdUp();
  if (!Dir.exists()) goto error;
  if (!Dir.mkdir(Name)) goto error;
  return true;
  error:
   int clicked = QMessageBox::critical( NULL, i18n("KNC Problem"), Message.sprintf(i18n("Unable to create Directory %s."), Name.data()),
                                       QMessageBox::Ok|QMessageBox::Default, QMessageBox::Retry, QMessageBox::Abort);

   if (clicked == QMessageBox::Retry) goto up;
   if (clicked == QMessageBox::Abort) {stop = true;return false;}
  return false;
}
bool FtpDirInfo::rename(ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& bStop)
{
 return _Plugin->rename(this, PDialog, Target, bOverwrite, bStop);
}
bool FtpDirInfo::remove(DeleteDialog* Dialog, bool& stop)
{
 return _Plugin->removeDirectory(this, Dialog, stop);
}


FtpLinkInfo::FtpLinkInfo(FtpPlugin* Plugin, QString Path, QString Name, double Size, QString Attrib, QString Owner, QString Group, QDateTime LastModified, QDateTime LastRead)
 : FtpFileInfo(Plugin, Path, Name, Size, Attrib, Owner, Group, LastModified, LastRead)
{
 int pos;
 if ((pos = Name.find("->")) != -1)
 {
   _Name = Name.left(pos-1);
   _PointsTo = Name.right(Name.length()-(pos+2));
 }
 if (_PointsTo.right(1) == "/") _bDirLink = true;
 else _bDirLink = false;
}

FtpLinkInfo::~FtpLinkInfo()
{}


bool FtpLinkInfo::copy(ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& bStop)
{
  //only follow files, not dirs
  if (!_bDirLink)
    return FtpFileInfo::copy(PDialog, Target, bOverwrite, bStop);
  return false;
}

bool FtpLinkInfo::rename(ProgressDialog* PFile, QString Target, bool& bOverwrite, bool& bStop)
{
  return false;
}

bool FtpLinkInfo::remove(DeleteDialog* PFile, bool& stop)
{
  return false;
}


QString FtpLinkInfo::infoAsString ()
{
 if (_bDirLink) return _Name + " DIR -> " + _PointsTo;
 else return _Name + " -> " + _PointsTo;
}

QString FtpFileInfo::getFile()
{
  return _Plugin->getFile(this);
}

QPixmap FtpLinkInfo::getIcon()
{
  QPixmap Icon;
  QPixmap Link = kapp->getIconLoader()->loadMiniIcon("link.xpm");
  if (_bDirLink)
    Icon = kapp->getIconLoader()->loadMiniIcon("folder.xpm");
  else
    Icon = kapp->getIconLoader()->loadMiniIcon("unknown.xpm");

  //this is a very ugly workaround -> it is not possible to make a deep copy of a pixmap!
  QImage Image = Icon.convertToImage();
  Icon.convertFromImage(Image);

  //add space for the Link in the mask of Icon
  QPainter P(Icon.mask());
  P.fillRect(0,8,8,8,color1);
  P.end();
  //now we can copy the Link into Icon
  bitBlt(&Icon, 0,8, &Link, 0,8,8,8, CopyROP, true);

  return Icon;
}

