/***************************************************************************
                          ftpfileinfo.h  -  description                              
                             -------------------                                         
    begin                : Sat Oct 16 1999                                           
    copyright            : (C) 1999 by Henrik Stormer                         
    email                : stormer@ifi.unizh.ch                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef FTPFILEINFO_H
#define FTPFILEINFO_H

#include <basefileinfo.h>

/**
  *@author Henrik Stormer
  */
class FtpPlugin;

class FtpFileInfo : public BaseFileInfo  {
public: 
	FtpFileInfo(FtpPlugin* Plugin, QString Path, QString Name, double Size, QString Attrib, QString Owner, QString Group, QDateTime LastModified, QDateTime LastRead);

	virtual ~FtpFileInfo();

 virtual long    getAttribute() const;
 virtual void    setAttribute(long Attribute);

 virtual QString infoAsString ();
 virtual BaseFileInfo* clone () {return new FtpFileInfo(*this);}

 virtual bool accessRead() const;
 virtual bool accessWrite() const;
 virtual bool accessExecute() const;

 virtual bool isDir() const {return false;}
 virtual bool isFile() const {return true;}

 virtual bool copy(ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& stop);
 virtual bool rename(ProgressDialog* PFile, QString Target, bool& bOverwrite, bool& stop);
 virtual bool remove(DeleteDialog* PFile, bool& stop);

 virtual QString getFile();

 protected:
  FtpPlugin* _Plugin;
 private:
  int _Mode;

};

class FtpDirInfo : public FtpFileInfo
{
  public:
	FtpDirInfo(FtpPlugin* Plugin, QString Path, QString Name, double Size, QString Attrib, QString Owner, QString Group, QDateTime LastModified, QDateTime LastRead);

	~FtpDirInfo();
  virtual bool isDir() const {return true;}
  virtual bool isFile() const {return false;}
  virtual QString getSize() const {return "<DIR>";}

  virtual BaseFileInfo* clone () {return new FtpDirInfo(*this);}
  QPixmap getIcon() {return kapp->getIconLoader()->loadMiniIcon("folder.xpm");}

  virtual bool copy(ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& stop);
  virtual bool rename(ProgressDialog* PFile, QString Target, bool& bOverwrite, bool& stop);
  virtual bool remove(DeleteDialog* PFile, bool& stop);
};

class FtpLinkInfo : public FtpFileInfo
{
  public:
	FtpLinkInfo(FtpPlugin* Plugin, QString Path, QString Name, double Size, QString Attrib, QString Owner, QString Group, QDateTime LastModified, QDateTime LastRead);

	~FtpLinkInfo();
  virtual bool isDir() const {return _bDirLink;}
  virtual bool isFile() const {return !_bDirLink;}
  virtual bool isLink() const {return true;}
//  virtual QString getSize() const {return "<DIR>";}
  virtual QString infoAsString ();
  virtual BaseFileInfo* clone () {return new FtpLinkInfo(*this);}
  QPixmap getIcon();

  virtual bool copy(ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& stop);
  virtual bool rename(ProgressDialog* PFile, QString Target, bool& bOverwrite, bool& stop);
  virtual bool remove(DeleteDialog* PFile, bool& stop);
  protected:
   QString _PointsTo;
   bool    _bDirLink;
};


#endif



















