/***************************************************************************
                          ftpplugin.cpp  -  description                              
                             -------------------                                         
    begin                : Sat Oct 16 1999                                           
    copyright            : (C) 1999 by Henrik Stormer                         
    email                : stormer@ifi.unizh.ch                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include "qmultilinedit.h"
#include "ftpplugin.h"
#include "kconfig.h"
#include "netinet/in.h"
#include <fcntl.h>
#include <errno.h>
#include "netdb.h"
#include "sys/socket.h"
#include "iostream.h"
#include "unistd.h"
//#include "mylistview.h"
#include "ftpfileinfo.h"
#include "qmessagebox.h"
#include "kapp.h"
#include "knc.h"
#include "splashscreen.h"
#include "helpfunc.h"
#include "ftppluginfactory.h"

void ShowMessageBox (QString Message, QString Param1 = "")
{
  int pos = Message.find("%s");
  if (pos != -1)
  {
    Message.remove(pos, 2);
    Message.insert(pos, Param1);
  }
  QMessageBox::information( NULL, i18n("KNC"), Message, QMessageBox::Ok|QMessageBox::Default);
}


bool FtpPlugin::checkTimeout()
{
  if (_CommandSocket == -1) return false;
  unsigned int len;
  int  val;
  if (getsockopt(_CommandSocket, SOL_SOCKET, SO_TYPE, &val, &len ) != 0) /*return false;*/
//  if (getsockopt(_CommandSocket, SOL_SOCKET, SO_TYPE, &i, &j) != 0) return false;
//  if (val != SOCK_STREAM)
  {
    ShowMessageBox(i18n("Socket no longer in use."));
    return false;
  }
  return true;
}

bool FtpPlugin::ripPath(QString Path, QString &FilePath, QString& SubPath)
{
  if (Path.left(6) == "ftp://")
  {
    Path.remove(0,6);
    int pos = Path.find("/");
    if (pos != -1)
    {
      FilePath = Path.left(pos);
      SubPath = Path.right(Path.length()-pos);
      return true;
    }
    return false;
  }
  else
  {
    return ::ripPath(Path, "#ftp:", FilePath, SubPath);
  }
  return false;
}

FtpPlugin::FtpPlugin(FtpPluginFactory* Factory, QMultiLineEdit* ProtocolView) : BasePlugin(),
   _Factory(Factory), _ProtocolView(ProtocolView),  _Anonymous(false), _CommandSocket(-1), _bConnect(false), _bEnding(true), _bActiveCommand(false), _bRefresh(false), _bFile(true), _OldDir("/")
{
}

FtpPlugin::FtpPlugin(FtpPluginFactory* Factory, QMultiLineEdit* ProtocolView,
            QString Host, QString Dir, QString User, QString Password, int Port, QString OldDir, QList<short>& ColumnOrder)
  : _Factory(Factory), _ProtocolView(ProtocolView),_Host(Host), _Dir(Dir), _User(User), _Password(Password), _Anonymous(false), _Port(Port),
    _CommandSocket(-1), _bConnect(false), _bEnding(true), _bActiveCommand(false), _bRefresh(false), _bFile(false), _OldDir(OldDir), _ColumnOrder(ColumnOrder)
{
  _FilePath = _Host;
}


FtpPlugin::~FtpPlugin()
{
 if (_bConnect) disconnect();
 _Factory->unregisterPlugin(this);
}

BasePluginFactory* FtpPlugin::getPluginFactory()
{
  return _Factory;
}

bool FtpPlugin::isResponsible(QString Path, QString SubPath)
{
  QString FirstPart, LastPart;
  if (!ripPath(Path, FirstPart, LastPart))
    FirstPart = Path+SubPath;

  if (FirstPart == _FilePath) return true;
  return false;
}



bool FtpPlugin::readConfig(QString Path)
{
  _ColumnOrder.clear();
  KConfig Config(Path);
  _Host = Config.readEntry("Host", "");
  _Dir = Config.readEntry("Dir", "/");
  _Port = Config.readNumEntry("Port", 21);
  _User = Config.readEntry("User", "anonymous");
  _Password = Config.readEntry("Password", "anonymous@anonymous.xxx.com");
 return true;
}

bool FtpPlugin::connect()
{
  struct sockaddr_in sin;
  struct hostent *hent;

  _ProtocolView->clear();

  sin.sin_port = htons(_Port);
  sin.sin_family = AF_INET;

  if ((hent = gethostbyname(_Host.data())) == NULL)
  {
    ShowMessageBox(i18n("Unable to get host: %s. Possible DNS failure."), _Host.data());
    return false;
  }
  memcpy((char *)&sin.sin_addr, hent->h_addr, hent->h_length);

  _CommandSocket = socket(PF_INET, SOCK_STREAM, 0); //IPPROTO_TCP);
  if (_CommandSocket == -1)
  {
    ShowMessageBox(i18n("Unable to create socket. Maybe a TCP/IP problem."));
    return false;
  }

  int on = 1;
  if (setsockopt(_CommandSocket, SOL_SOCKET,SO_REUSEADDR | SO_KEEPALIVE,
		   (char*)&on, sizeof(on)) == -1)
  {
     ShowMessageBox(i18n("Unable to change socket options. Maybe you have to wait 2 minutes."));
     close (_CommandSocket);
     return false;
  }
  fcntl(_CommandSocket, F_SETFL,(fcntl(_CommandSocket,F_GETFL)|O_NONBLOCK));
  if ( ::connect(_CommandSocket, (struct sockaddr *)&sin, sizeof(sin)) == -1)
  {
    if (errno != EINPROGRESS && errno != EWOULDBLOCK) return false;
  }

  if (!getResponse('2'))
  {
    ShowMessageBox(i18n("Unable to establish connection. Error: %s"), _LastLine);
    close (_CommandSocket);
    _CommandSocket = -1;
    return false;
  }
  return true;
}

bool FtpPlugin::createDataPort()
{
    union {
		struct sockaddr sa;
		struct sockaddr_in in;
    } sin;
    struct linger lng = { 0, 0 };
    ksize_t l;
    char buf[64];

    l = sizeof(sin);
    if (getsockname(_CommandSocket,&sin.sa,&l) < 0) return false;

    _DataSocket = socket(PF_INET,SOCK_STREAM,IPPROTO_TCP);
    if (_DataSocket == -1)  return false;

    int on=1;
    if (setsockopt(_DataSocket,SOL_SOCKET,SO_REUSEADDR,
		   (char*)&on,sizeof(on)) == -1) goto err;

    if (setsockopt(_DataSocket,SOL_SOCKET,SO_LINGER,
		   (char*)&lng,sizeof(lng)) == -1) goto err;

    sin.in.sin_port = 0;
    if (bind(_DataSocket,&sin.sa,sizeof(sin)) == -1) goto err;

    if (listen(_DataSocket,1) < 0) goto err;

    if (getsockname(_DataSocket, &sin.sa,&l) < 0) goto err;

    sprintf(buf,"port %d,%d,%d,%d,%d,%d",
	    (unsigned char)sin.sa.sa_data[2],(unsigned char)sin.sa.sa_data[3],
	    (unsigned char)sin.sa.sa_data[4],(unsigned char)sin.sa.sa_data[5],
	    (unsigned char)sin.sa.sa_data[0],(unsigned char)sin.sa.sa_data[1]);
    if (!sendCommand(buf, '2')) goto err;
  return true;
 err:
  ShowMessageBox(i18n("Unable to create data port. Error: %s "), _LastLine );
  close (_DataSocket);
  _DataSocket = -1;
  return false;
}

bool FtpPlugin::login()
{
    QString Command("user ");
    if (!_Anonymous)  Command += _User;
    else Command += "FTP";
    if (!sendCommand(Command,'3'))
    {
     	if (_LastLine[0]=='2') return true; // no password required
      else
      {
        ShowMessageBox(i18n("Login fails: %s"), _LastLine);
      	return false;
      }
    }
    Command = "pass ";
    if (!_Anonymous) Command += _Password;
    else Command += "anonymous@xxx.com";

    if (sendCommand( Command, '2' ))
    {
        // "drin, so leicht geht das..."
        if( sendCommand( "syst", '2' ) )
            if( !strncmp( _LastLine, "215 Windows_NT version", 22 ) ) // will do for any version
                sendCommand( "site dirstyle", '2' );
        return true;
    }
    else
    {
        ShowMessageBox(i18n("Login fails. Possibly wrong username or password: %s"), _LastLine);
      	return false;
    }

  return false;
}

QString FtpPlugin::readLine()
{
 char ResponseBuffer[2048];
 int num;
 if (_Buffer.length() == 0)
 {
   while ((num = read(_CommandSocket, ResponseBuffer, sizeof(ResponseBuffer)-1)) > 0)
   {
     ResponseBuffer[num] = '\0';
     _Buffer += ResponseBuffer;
   }
 }
 int pos = _Buffer.find("\r\n");
 if (pos != -1)
 {
   QString Line = _Buffer.left(pos);
   _Buffer = _Buffer.right(_Buffer.length()-(pos+2));
   _ProtocolView->insertLine(Line);
   _ProtocolView->setCursorPosition(_ProtocolView->numLines(), 0);
   return Line;
 }
 return NULL;
}

bool FtpPlugin::getResponse(char FirstNumber)
{
 bool Res = false;
 int count = 0;
 int waitsec = 0;
 while(count < 50)
 {
   _LastLine = readLine();
   if (_LastLine == NULL)
   {
     sleep(waitsec); count++;
     if (count == 20) waitsec++;
   }
   else count = 50;
 }
 if (_LastLine == NULL)
 {
   ShowMessageBox(i18n("The FTP connection could not be established, because of a timeout."));
   return false;
 }
 if (_LastLine[0] == '1') _bEnding = false;
 if (_LastLine[0] == FirstNumber) Res = true;
 if (_LastLine[3] == '-')
 {
   QString Match = _LastLine.left(4);
   Match[3] = ' ';
   QString NextLine;
   do
   {
     NextLine = readLine();
     if (NextLine != "")
     {
       _LastLine += NextLine.right(NextLine.length()-4);
     }
   }
   while (Match != NextLine.left(4));
 }
 return Res;
}


bool FtpPlugin::sendCommand(QString Command, char FirstNumber)
{
 _ProtocolView->insertLine(Command);
 _ProtocolView->setCursorPosition(_ProtocolView->numLines(),0);
 Command += "\r\n";

 if (!_bEnding)
 {
   _bEnding = true;
   if (!getResponse('2')) return false;
 }
 if (write(_CommandSocket, Command.data(), Command.length()) <= 0)
 {
   //if write fails, the connection is broken!
   _CommandSocket = -1;
   _bConnect = false;
   return false;
 }
 else
 {
   return getResponse(FirstNumber);
 }
}

bool FtpPlugin::acceptConnect()
{
  struct sockaddr addr;
  ksize_t l;
  fd_set mask;

  FD_ZERO(&mask);
  FD_SET(_DataSocket,&mask);

  if (select( _DataSocket + 1, &mask,NULL,NULL, 0L) == 0) goto err;

  l = sizeof(addr);
  if ((_UsedDataSocket = accept(_DataSocket,&addr,&l)) > 0)
  {
//    _bActiveCommand = true;
    return true;
  }

  return false;
  err:
   ShowMessageBox(i18n("Failed to accept data connection"));
   close(_DataSocket);
   return false;
}


char	*monthes[] = {"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};

int getMonthNumber(const QString& Line, int& pos)
{
	int	res = 0;
  while ((res < 12) && ((pos = Line.find(monthes[res])) == -1)) res++;
	return (res != 12 ? (res+1) : -1);
}

BaseFileInfo* FtpPlugin::generateEntry(QString Path, QString Line)
{
  BaseFileInfo* Entry;
  if (Line.length() < 15) return NULL;
  QString Contents [ColumnSize];
  char fType = Line[0];

  //don't show devices (fType == 'c'|'b') or other problematic things,
  //because the ftp server adds strange columns for it
  if ((fType != 'l') && (fType != 'd') && (fType != 'f') && (fType != '-')) return NULL;

  QListIterator<short> it(_ColumnOrder);
  for (;it.current();++it)
  {
    QString Tokenizer;
    if (it.atLast()) Tokenizer = "\0"; else Tokenizer = " ";
    int index = *(it.current());
    if (it.atFirst()) Contents[index] = strtok(const_cast<char*>(Line.data()), Tokenizer);
    else              Contents[index] = strtok(NULL, Tokenizer);
  }
  Contents[Column_Name] = Contents[Column_Name].stripWhiteSpace();

  int pos;
  int Month = getMonthNumber(Contents[Column_Date1], pos);
  int Day = Contents[Column_Date2].toInt();
  int Year;
  QString SHour, SMinute;
  int Hour=0, Minute=0;
  if (Contents[Column_Date3].contains(":"))
  {
    Year = QDateTime::currentDateTime().date().year();
    SHour = strtok(const_cast<char*>(Contents[Column_Date3].data()), ":");
    SMinute = strtok(NULL, "\0");
    Hour = SHour.toInt();
    Minute = SMinute.toInt();
  }
  else
    Year = Contents[Column_Date3].toInt();

  Contents[Column_Attrib].remove(0,1);
  QDateTime DT (QDate(Year, Month, Day), QTime(Hour, Minute));
  if (Contents[Column_Name] == ".") return NULL;
  long dSize = Contents[Column_Size].toLong();


  if (fType == 'd') Entry = new FtpDirInfo(this, Path, Contents[Column_Name], dSize, Contents[Column_Attrib], Contents[Column_Owner], Contents[Column_Group], DT, DT);
  else if (fType == 'l') Entry = new FtpLinkInfo(this, Path, Contents[Column_Name], dSize, Contents[Column_Attrib], Contents[Column_Owner], Contents[Column_Group], DT, DT);
       else              Entry = new FtpFileInfo(this, Path, Contents[Column_Name], dSize, Contents[Column_Attrib], Contents[Column_Owner], Contents[Column_Group], DT, DT);
  return Entry;
}

bool FtpPlugin::gotoSubDir(QString SubDir)
{
  QString Command("cwd ");
  Command +=  SubDir;
  return sendCommand(Command,'2');
}

bool FtpPlugin::create(QString Path, MyListView* ListBox, QString CurrentItemName, bool ShowHiddenFiles)
{
  if (!_bConnect)
  {
    SplashScreen Splash(0,0,0,i18n("One moment, try to connect..."));
    Splash.show();
    qApp->processEvents(2000);

    if (Path.left(6) == "ftp://") _bFile = false;
    else _bFile = true;

    if (_bFile) //read configuration from file
    {
       QString FilePath, SubPath;
       ripPath(Path, FilePath, SubPath);
       _FilePath = FilePath;
       if (SubPath != "") _Dir = SubPath;
       if (!readConfig(FilePath)) return false;
    }
    else //read configuration from saved ftp configurations
    {
       QString Server, SubDir;
       ripPath(Path, Server, SubDir);
       FtpConfiguration* Config = ((FtpPluginFactory*)getPluginFactory())->getFtpConfiguration(Server);
       if (!Config)
       {
         ShowMessageBox(i18n("Unable to read the configuration for FTP Host: %s "), Server);
         return false;
       }
       _Host = Config->_Host;
       _FilePath = _Host;
       _Anonymous = Config->_Anonymous;
       if (!_Anonymous)
       {
         _User = Config->_User;
         _Password = Config->_Password;
       }
       _Port = Config->_Port;
       _Dir = Config->_Dir;
       if (SubDir != "/") _Dir = SubDir;

       _ColumnOrder.clear();
       QListIterator<short> it(Config->_ColumnOrder);
       for (;it.current();++it)
       {
         cout <<*it.current() << endl;
         _ColumnOrder.append(new short(*it.current()));
       }
    }
    if (!connect()) return false;
    if (!login()) return false;
    _bConnect = true;
    if (!gotoSubDir(_Dir))
    {
      QString Message = i18n("Unable to change to directory: ") + _Dir;
      ShowMessageBox(Message + "Error: %s", _LastLine);
      return false;
    }
    Splash.close();
  }
//  else
//    if (!checkTimeout()) return false;

  ListBox->clear();

  if (!createDataPort()) return false;

  if (!sendCommand("LIST",'1')) return false;

  if (!acceptConnect()) return false;

  QString TBuffer, Line;
  char Buffer[2048];
  QString StrBuffer;
  int n = sizeof(Buffer)-1;
  while (n > 0)
  {
    if (StrBuffer.length() == 0)
    {
      n = read(_UsedDataSocket, Buffer, sizeof(Buffer)-1);
      Buffer[n] = '\0';
      StrBuffer = Buffer;
    }
    int pos = StrBuffer.find("\n");

    if (pos != -1)
    {
      QString Line = StrBuffer.left(pos);
      StrBuffer = StrBuffer.right(StrBuffer.length()-(pos+1));
      BaseFileInfo* Info = generateEntry(Path,Line);
      if (Info)
      {
        MyListViewItem* Item =  ListBox->insertItem(Info);
        if ((Item) && (Item->getFileInfo()->fileName() == CurrentItemName)) ListBox->setCurrentItem(Item);
      }
    }
    else
    {
      n = read(_UsedDataSocket, Buffer, sizeof(Buffer)-1);
      Buffer[n] = '\0';
      StrBuffer += Buffer;
    }
  }
  close(_DataSocket);
  close(_UsedDataSocket);
  return true;
}

void FtpPlugin::disconnect()
{
  QString Message;
  if (!sendCommand("QUIT",'2'))
     ShowMessageBox(i18n("Unable to quit connection. Error: %s"), _LastLine);

	if (_CommandSocket != -1) ::close(_CommandSocket);
  _CommandSocket = -1;
  _bConnect = false;
	return;
}

QString FtpPlugin::cdUp(QString Path, QString& Name)
{
   QString Message;
   QString FilePath, SubPath;
   ripPath(Path, FilePath, SubPath);
   if (SubPath == "/")
   {
     if (_bFile)
     {
       cutUpPath(FilePath, FilePath, Name);
       disconnect();
       return FilePath;
     }
     else
     {
       disconnect();
       return _OldDir;
     }
   }
   else
   {
     QString Command("cwd ");
     cutUpPath(SubPath, SubPath, Name);
     Command += SubPath;
     if (!sendCommand(Command,'2'))
     {
       ShowMessageBox(i18n("Unable to change directory. Error: %s"), _LastLine);
       return Path;
     }
     if (_bFile)
       return FilePath + "#ftp:" + SubPath;
     else
       return "ftp://" + FilePath + SubPath;
   }
   return Path;
}

QString FtpPlugin::cdDown(QString Path, QString DownPath)
{
  QString Message;
  if ((Path.find("#ftp:") != -1) || (Path.left(6) == "ftp://"))
  {
     QString Command("cwd ");
     Command +=  DownPath;
     if (!sendCommand(Command,'2'))
     {
       QString Message = i18n("Unable to change to directory: ") + DownPath;
       ShowMessageBox(Message + " Error: %s", _LastLine);
       return Path;
     }
     return Path + DownPath + "/";
  }
  else
  {
    return Path + DownPath + "#ftp:/";
  }
}

void FtpPlugin::recurseSubdir(QString Dir, QList<BaseFileInfo>& FileList, double& countFiles, double& sizeAll)
{
  QString SubDir, Dummy;
  ripPath(Dir, Dummy, SubDir);
  QString OldDir;
  if (!sendCommand("pwd",'2'))
  {
    ShowMessageBox(i18n("Unable to query directory. Error: %s "), _LastLine);
    return;
  }
  else
  {
    int lpos = _LastLine.find("\"");
    if (lpos != -1)
    {
      int rpos = _LastLine.find("\"", lpos + 1);
      OldDir = _LastLine.mid(lpos+1, rpos - lpos - 1);
    }
    else
    {
      ShowMessageBox(i18n("Error while querying directory."));
      return;
    }
  }

  if (!gotoSubDir(SubDir)) return;

  if (!createDataPort())
  {
    return;
  }
  if (!sendCommand("LIST",'1'))
  {
    ShowMessageBox(i18n("Unable to send Command: LIST  Error: %s"), _LastLine);
    return;
  }
  if (!acceptConnect())
  {
    return;
  }

  QString TBuffer, Line;
  char Buffer[2048];
  QString StrBuffer;
  QList<BaseFileInfo> AllDirs, AllFiles;
  int n = sizeof(Buffer)-1;
  while (n > 0)
  {
    if (StrBuffer.length() == 0)
    {
      n = read(_UsedDataSocket, Buffer, sizeof(Buffer)-1);
      Buffer[n] = '\0';
      StrBuffer = Buffer;
    }
    int pos = StrBuffer.find("\n");
    if (pos != -1)
    {
      QString Line = StrBuffer.left(pos);
      StrBuffer = StrBuffer.right(StrBuffer.length()-(pos+1));
      BaseFileInfo* Info = generateEntry(Dir, Line);
      if (Info)
      {
        if (Info->isDir())
        {
          if ((Info->fileName() != ".") && (Info->fileName() != ".."))   AllDirs.append(Info);
        }
        if (Info->isFile())
          AllFiles.append(Info);
      }
    }
    else
    {
      n = read(_UsedDataSocket, Buffer, sizeof(Buffer)-1);
      Buffer[n] = '\0';
      StrBuffer += Buffer;
    }
  }
  close(_DataSocket);
  close(_UsedDataSocket);

  for (QListIterator<BaseFileInfo>it(AllDirs);it.current();++it)
  {
     FileList.append(it.current());
     recurseSubdir(Dir+it.current()->fileName() + "/", FileList, countFiles, sizeAll);
  }
  for (QListIterator<BaseFileInfo>it(AllFiles);it.current();++it)
  {
     FileList.append(it.current());
     sizeAll += it.current()->size();
     countFiles++;
  }
  if (!sendCommand("cwd " + OldDir,'2'))
  {
    ShowMessageBox(i18n("Unable to change directory.  Error: %s"), _LastLine);
    return;
  }
}


//we only refresh, this is easier than insert all items one by one. Therefore
//source and target as well as insert and remove do the same.
void FtpPlugin::sourceAfterFileOperation(QString Path, int Operation)
{
 targetAfterFileOperation(Path, Operation);
}
void FtpPlugin::targetAfterFileOperation(QString Path, int Operation)
{
  if (_bRefresh)
  {
    QListIterator<MyListView> it (_refreshListBox);
    for (;it.current();++it) it.current()->refresh();

    _refreshListBox.clear();
    _bRefresh = false;
  }
}

void FtpPlugin::insertEntry(MyListView* ListBox, QString Dir, QString Name)
{
  if (ListBox->getDir() == Dir)
  {
    _bRefresh = true;
    if (_refreshListBox.find(ListBox) == -1) _refreshListBox.append(ListBox);
  }
}

void FtpPlugin::removeEntry(MyListView* ListBox, QString Dir, QString Name)
{
  insertEntry(ListBox, Dir, Name);
}


bool FtpPlugin::copyIn(ProgressDialog* PDialog, ST st, bool& bOverwrite, bool& bStop)
{
  if (st.Info->isDir())
  {
    QString Path, Dir;
    cutUpPath(st.Target, Path, Dir);
    return makeDirectory(Path, Dir);
  }
  else
    return uploadFile(PDialog, st, bOverwrite, bStop);
}

bool FtpPlugin::uploadFile(ProgressDialog* PDialog, ST st, bool& bOverwrite, bool& bStop)
{
	if (!sendCommand("TYPE I",'2'))
  {
    ShowMessageBox(i18n("Unable to set to binary mode.  Error: %s"), _LastLine);
		return false;
	}
  if (!createDataPort())
  {
//    ShowMessageBox(i18n("Unable to create data port."));
    return false;
  }
  QString Command = "stor ";
  QString FilePath, Dummy;
  ripPath(st.Target, Dummy, FilePath);
  Command += FilePath;
  up1:
	if (!sendCommand(Command,'1'))
  {
     QString Message;
     int clicked = QMessageBox::critical( NULL, i18n("KNC Problem"),
                                          Message.sprintf(i18n("Unable to store a file. Error: %s"), _LastLine.data()),
                                          QMessageBox::Ok|QMessageBox::Default, QMessageBox::Retry, QMessageBox::Abort);

     if (clicked == QMessageBox::Retry) goto up1;
     if (clicked == QMessageBox::Abort) {bStop = true;}

     close(_DataSocket);
	   return false;
	}
  up2:
  if (!acceptConnect())
  {
    QString Message;
    int clicked = QMessageBox::critical( NULL, i18n("KNC Problem"),
                                         Message.sprintf(i18n("Unable to accept connection. Error: %s"), _LastLine.data()),
                                         QMessageBox::Ok|QMessageBox::Default, QMessageBox::Retry, QMessageBox::Abort);

    if (clicked == QMessageBox::Retry) goto up2;
    if (clicked == QMessageBox::Abort) {bStop = true;}

    close(_DataSocket);
    return false;
  }
  up3:
	QFile File (st.Source);
  if (!File.open(IO_ReadOnly))
  {
    QString Message;
    int clicked = QMessageBox::critical( NULL, i18n("KNC Problem"),
                                         Message.sprintf(i18n("Unable to open file: %s"), st.Source.data()),
                                         QMessageBox::Ok|QMessageBox::Default, QMessageBox::Retry, QMessageBox::Abort);

    if (clicked == QMessageBox::Retry) goto up3;
    if (clicked == QMessageBox::Abort) {bStop = true;}

    close(_DataSocket);
    return false;
	}
  char Buffer[2048];
  int n;
  while ((n = File.readBlock(Buffer, sizeof(Buffer))) > 0)
  {
    write(_UsedDataSocket, Buffer, n);
    if (PDialog) PDialog->addSize(n);
  }
  File.close();
  close(_DataSocket);
  close(_UsedDataSocket);
  return true;
}


bool FtpPlugin::downloadFile(FtpFileInfo* Info, ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& bStop)
{
	QFile File(Target);
  if ((File.exists()) && (!bOverwrite))
  {
    QString Message;
    QMessageBox mb( "KNC Warning", Message.sprintf(i18n("File %s exists. Overwrite?"), Target.data()),QMessageBox::Critical,
                     QMessageBox::Yes | QMessageBox::Default, QMessageBox::No, QMessageBox::Retry );
     mb.setButtonText( QMessageBox::Retry, "All" );
     switch( mb.exec() ) {case QMessageBox::No: return false; break;
       case QMessageBox::Retry: bOverwrite = true; break;}
  }
  up1:
  if (!File.open(IO_WriteOnly))
  {
    QString Message;
    int clicked = QMessageBox::critical( NULL, i18n("KNC Problem"),
                                         Message.sprintf(i18n("Unable to open file %s"), Target.data()),
                                         QMessageBox::Ok|QMessageBox::Default, QMessageBox::Retry, QMessageBox::Abort);

    if (clicked == QMessageBox::Retry) goto up1;
    if (clicked == QMessageBox::Abort) {bStop = true;}
		return false;
	}
	if (!sendCommand("TYPE I",'2'))
  {
    ShowMessageBox(i18n("Unable to set to binary mode. Error: %s"), _LastLine);
		return false;
	}
  if (!createDataPort())
  {
    return false;
  }
  QString Command = "retr ";
  QString Dummy, SubDir;
  ripPath(Info->filePath(), Dummy, SubDir);
  Command += SubDir;
  up2:
  if (!sendCommand(Command,'1'))
  {
    QString Message;
    int clicked = QMessageBox::critical( NULL, i18n("KNC Problem"),
                                         Message.sprintf(i18n("Unable to retrieve file. Error: %s"), _LastLine.data()),
                                         QMessageBox::Ok|QMessageBox::Default, QMessageBox::Retry, QMessageBox::Abort);

    if (clicked == QMessageBox::Retry) goto up2;
    if (clicked == QMessageBox::Abort) {bStop = true;}

    close(_DataSocket);
    return false;
    return false;
  }
  if (!acceptConnect())
  {
    return false;
  }

  char Buffer[2048];
  int n;
  while ((n = read(_UsedDataSocket, Buffer, sizeof(Buffer))) > 0)
  {
    if (PDialog) PDialog->addSize(n);
    File.writeBlock(Buffer, n);
  }
  File.close();

  close(_DataSocket);
  close(_UsedDataSocket);
  return true;
}

bool FtpPlugin::makeDirectory(QString Path, QString Dir)
{
  QString Command("mkd ");
  QString Dummy, SubDir;
  ripPath(Path, Dummy, SubDir);
  Command += SubDir + Dir;
  if (!sendCommand(Command,'2'))
  {
    QString Message = i18n("Unable to create directory: ") + Dir;
    ShowMessageBox(Message + " Error %s", _LastLine);
    return false;
  }
  return true;
}


QString FtpPlugin::getFile(FtpFileInfo* Info)
{
  bool dummy1 = true;
  bool dummy2 = false;
  QString TempFileName = tmpnam (NULL);
  if (!downloadFile(Info, NULL, TempFileName, dummy1, dummy2)) return "";
  else return TempFileName;
}

bool FtpPlugin::removeDirectory(FtpDirInfo* Info, DeleteDialog* Dialog, bool& bStop)
{
  QString Command("rmd ");
  QString Dummy, SubDir;
  ripPath(Info->filePath(), Dummy, SubDir);
  Command += SubDir;
  up:
  if (!sendCommand(Command,'2'))
  {
    QString Message;
    int clicked = QMessageBox::critical( NULL, i18n("KNC Problem"),
                                         Message.sprintf(i18n("Unable to remove directory. Error: %s"), _LastLine.data()),
                                         QMessageBox::Ok|QMessageBox::Default, QMessageBox::Retry, QMessageBox::Abort);

    if (clicked == QMessageBox::Retry) goto up;
    if (clicked == QMessageBox::Abort) {bStop = true;}
    return false;
  }
  return true;
}

bool FtpPlugin::removeFile(FtpFileInfo* Info, DeleteDialog* Dialog, bool& bStop)
{
  QString Command("dele ");
  QString Dummy, SubDir;
  ripPath(Info->filePath(), Dummy, SubDir);
  Command += SubDir;
  up:
  if (!sendCommand(Command,'2'))
  {
    QString Message;
    int clicked = QMessageBox::critical( NULL, i18n("KNC Problem"),
                                         Message.sprintf(i18n("Unable to remove file. Error: %s"), _LastLine.data()),
                                         QMessageBox::Ok|QMessageBox::Default, QMessageBox::Retry, QMessageBox::Abort);

    if (clicked == QMessageBox::Retry) goto up;
    if (clicked == QMessageBox::Abort) {bStop = true;}
    return false;
  }
  return true;
}

void FtpPlugin::setAttribute(FtpFileInfo* Info, long Attribute)
{
  QString Command("site chmod ");
  QString Attrib;
  long A;
  A = 0;
  if (Attribute & QFileInfo::ReadUser) A |= 1;
  if (Attribute & QFileInfo::WriteUser) A |= 2;
  if (Attribute & QFileInfo::ExeUser) A |= 4;
  Attrib = QString().setNum(A);
  A = 0;
  if (Attribute & QFileInfo::ReadGroup) A |= 1;
  if (Attribute & QFileInfo::WriteGroup) A |= 2;
  if (Attribute & QFileInfo::ExeGroup) A |= 4;
  Attrib += QString().setNum(A);
  A = 0;
  if (Attribute & QFileInfo::ReadOther) A |= 1;
  if (Attribute & QFileInfo::WriteOther) A |= 2;
  if (Attribute & QFileInfo::ExeOther) A |= 4;
  Attrib += QString().setNum(A);

  QString Dummy, SubDir;
  ripPath(Info->filePath(), Dummy, SubDir);

  Command += Attrib + " " + SubDir;
  if (!sendCommand(Command,'2'))
  {
    QString Message = i18n("Unable to change attribute of file: %s") + Info->fileName();
    ShowMessageBox(Message + " Error: %s ", _LastLine);
 }
}



bool FtpPlugin::rename(FtpFileInfo* Info, ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& bStop)
{
  QString Command = "rnfr ";
  QString Dummy, SubDir;
  ripPath(Info->filePath(), Dummy, SubDir);
  Command += SubDir;
  up:
  if (!sendCommand(Command,'3'))
  {
    QString Message;
    int clicked = QMessageBox::critical( NULL, i18n("KNC Problem"),
                                         Message.sprintf(i18n("Unable to rename. Error: %s"), _LastLine.data()),
                                         QMessageBox::Ok|QMessageBox::Default, QMessageBox::Retry, QMessageBox::Abort);

    if (clicked == QMessageBox::Retry) goto up;
    if (clicked == QMessageBox::Abort) {bStop = true;}
    return false;
  }
  Command = "rnto ";
  ripPath(Target, Dummy, SubDir);
  Command += SubDir;
  if (!sendCommand(Command,'2'))
  {
    QString Message;
    int clicked = QMessageBox::critical( NULL, i18n("KNC Problem"),
                                         Message.sprintf(i18n("Unable to rename. Error: %s"), _LastLine.data()),
                                         QMessageBox::Ok|QMessageBox::Default, QMessageBox::Retry, QMessageBox::Abort);

    if (clicked == QMessageBox::Retry) goto up;
    if (clicked == QMessageBox::Abort) {bStop = true;}
    return false;
  }
  if (PDialog) PDialog->addSize(Info->size());
  return true;
}











