/***************************************************************************
                          ftpplugin.h  -  description                              
                             -------------------                                         
    begin                : Sat Oct 16 1999                                           
    copyright            : (C) 1999 by Henrik Stormer                         
    email                : stormer@ifi.unizh.ch                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef FTPPLUGIN_H
#define FTPPLUGIN_H

#include <baseplugin.h>
#include <basepluginfactory.h>
/**
  *@author Henrik Stormer
  */

#include <mylistview.h>

class MyListViewItem;
class FtpFileInfo;
class FtpDirInfo;
class QMultiLineEdit;
class FtpPluginFactory;

class FtpPlugin : public BasePlugin  {
  friend FtpPluginFactory;
public: 
	FtpPlugin(FtpPluginFactory* Factory, QMultiLineEdit* ProtocolView);
  FtpPlugin(FtpPluginFactory* Factory, QMultiLineEdit* ProtocolView,
            QString Host, QString Dir, QString User, QString Password, int Port, QString OldDir, QList<short>& ColumnOrder);
	~FtpPlugin();
  virtual BasePluginFactory* getPluginFactory();

  virtual bool isResponsible(QString Path, QString SubPath);
  virtual bool create(QString Path, MyListView* ListBox, QString CurrentItemName, bool ShowHiddenFiles);
  virtual QString cdUp(QString Path, QString& Name);
  virtual QString cdDown(QString Path, QString DownPath);

  virtual void recurseSubdir(QString Dir, QList<BaseFileInfo>& FileList, double& countFiles, double& sizeAll);
  virtual bool makeDirectory(QString Path, QString Dir);
  virtual void insertEntry(MyListView* ListBox, QString Dir, QString Name);
  virtual void removeEntry(MyListView* ListBox, QString Dir, QString Name);

  virtual void sourceAfterFileOperation(QString Path, int FO);
  virtual void targetAfterFileOperation(QString Path, int FO);

  virtual bool copyIn(ProgressDialog* PDialog, ST st, bool& bOverwrite, bool& bStop);
  bool uploadFile(ProgressDialog* PDialog, ST st, bool& bOverwrite, bool& bStop);
  bool downloadFile(FtpFileInfo* Info, ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& stop);
  bool rename(FtpFileInfo* Info, ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& stop);

  QString getFile(FtpFileInfo*);
  bool removeFile(FtpFileInfo* Info, DeleteDialog* Dialog, bool& bStop);
  bool removeDirectory(FtpDirInfo* Info, DeleteDialog* Dialog, bool& bStop);
  void setAttribute(FtpFileInfo*, long Attribute);
  virtual QString getType() {return "FtpPlugin";}
  virtual QString getHost() {return _Host;}
  virtual FtpPlugin* clone(QString Dir, QString SubDir) {FtpPlugin* P = new FtpPlugin(*this);P->_FilePath = Dir+SubDir;return P;}
 protected:
  bool            checkTimeout();
  bool            ripPath(QString Path, QString &FilePath, QString& SubPath);
  bool            connect();
  void            disconnect();
  bool            acceptConnect();
  bool            login();
  bool            readConfig(QString Path);
  bool            createDataPort();
  bool            sendCommand(QString Command, char FirstNumber);
  QString         readLine();
  bool            getResponse(char FirstNumber);
  BaseFileInfo*   generateEntry(QString Path, QString Line);
  bool            gotoSubDir(QString SubDir);
  void            getsActive(MyListView* ListBox) {_ActListBox = ListBox;}
 private:
  FtpPluginFactory*  _Factory;
  QMultiLineEdit*    _ProtocolView;  QString            _FilePath;
  QString            _Host, _Dir, _User, _Password;
  bool               _Anonymous;
  QString            _Buffer, _LastLine;
  int                _Port;
  int                _DataSocket, _CommandSocket, _UsedDataSocket;
  bool               _bConnect, _bEnding, _bActiveCommand;
  QList<MyListView>  _refreshListBox;
  bool               _bRefresh;
  int                _beginPosition[7];
  bool               _stablePosition;
  bool               _bFile;
  QString            _OldDir;
  MyListView*        _ActListBox;
  QList<short>       _ColumnOrder;
};




#endif

















































