/***************************************************************************
                          myfileinfo.cpp  -  description
                             -------------------
    begin                : Sat Sep 25 1999
    copyright            : (C) 1999 by Henrik Stormer
    email                : stormer@ifi.unizh.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include <iostream.h>
#include "generalfileinfo.h"
#include <qmessagebox.h>
#include <qdir.h>
#include <kapp.h>
#include <knc.h>
#include "time.h"
#include <qbitmap.h>
#include <qimage.h>
#include "helpfunc.h"
#include <pwd.h>
#include <grp.h>

const int BEST_BUFFERSIZE = 8192;

GeneralFileInfo::GeneralFileInfo()
{
  _dSize = 0;
  _Mode = 0;
}

GeneralFileInfo::GeneralFileInfo(QString Path, QString Name, struct stat& st)
{
 _Mode = st.st_mode;
 _Name = Name;
 _Path = Path;
 if (_Path.right(1) != "/") _Path += "/";
 _dSize = st.st_size;

 _UserId = st.st_uid;
 _GroupId = st.st_gid;
 struct passwd * user = getpwuid( _UserId );
 if (user) _Owner = user->pw_name;
 struct group * gr = getgrgid( _GroupId );
 if (gr) _Group = gr->gr_name;

 //this is Y2K conform because tmt->tm_year counts the years since 1900!
 struct tm* tmt = localtime(&st.st_atime);
 _LastRead.setDate(QDate(1900+tmt->tm_year, 1+tmt->tm_mon, tmt->tm_mday));
 _LastRead.setTime(QTime(tmt->tm_hour, tmt->tm_min, tmt->tm_sec));
 tmt = localtime(&st.st_mtime);
 _LastModified.setDate(QDate(1900+tmt->tm_year, 1+tmt->tm_mon, tmt->tm_mday));
 _LastModified.setTime(QTime(tmt->tm_hour, tmt->tm_min, tmt->tm_sec));
}


GeneralFileInfo::~GeneralFileInfo()
{}

QPixmap GeneralFileInfo::getIcon()
{
  QString Icon = "unknown.xpm";
  if (((_Mode&S_IXUSR)||(_Mode&S_IXGRP)||(_Mode&S_IXOTH))&&(!isLink())) Icon = "exec.xpm";
  if ((fileName().right(4) == ".tgz") || (fileName().right(7) == ".tar.gz")
      ||(fileName().right(4) == ".TGZ") || (fileName().right(7) == "TAR.GZ")
      ||(fileName().right(4) == ".tar") || (fileName().right(4) == ".TAR"))
    Icon = "tgz.xpm";

  if (fileName().right(4) == ".ftp") Icon = "ftp.xpm";
  return kapp->getIconLoader()->loadMiniIcon(Icon);
}

QString GeneralFileInfo::getAttrib() const
{
  QString Attrib;

  if (S_IRUSR & _Mode) Attrib += "r"; else Attrib += "-";
  if (S_IWUSR & _Mode) Attrib += "w"; else Attrib += "-";
  if (S_IXUSR & _Mode) Attrib += "x"; else Attrib += "-";

  if (S_IRGRP & _Mode) Attrib += "r"; else Attrib += "-";
  if (S_IWGRP & _Mode) Attrib += "w"; else Attrib += "-";
  if (S_IXGRP & _Mode) Attrib += "x"; else Attrib += "-";

  if (S_IROTH & _Mode) Attrib += "r"; else Attrib += "-";
  if (S_IWOTH & _Mode) Attrib += "w"; else Attrib += "-";
  if (S_IXOTH & _Mode) Attrib += "x"; else Attrib += "-";

  return Attrib;
}

QString GeneralDirInfo::getSize() const
{
 QString Help;
 if (!_bSizeSet) return "<DIR>";
 return Help.setNum(_dSize, 'g', 25);
}


QString GeneralFileInfo :: infoAsString ()
{
 QString Help;
 return fileName() + ";   " + getSize() + ";   " + getAttrib();
}

GeneralLinkInfo :: GeneralLinkInfo(QString Path, QString Name, struct stat& st)
 : GeneralFileInfo(Path, Name, st)
{
  char buf[2048];
  int n = readlink(Path+Name, buf, 2048);
  buf[n] = '\0';
  _Link = buf;
  struct stat nst;
  stat(Path+Name, &nst);
  if (S_ISDIR(nst.st_mode))  _bFileLink = false;
    else _bFileLink = true;
}

QString GeneralLinkInfo :: infoAsString ()
{
  return fileName() + " -> " + getLink();
}

void GeneralFileInfo::setAttribute(long Attrib)
{
  if (chmod (filePath(), Attrib) == -1)
    QMessageBox::information( NULL, i18n("KNC Problem"), i18n("Unable to change attributes of file: ") + filePath(),
                                         QMessageBox::Ok| QMessageBox::Default );

}

long GeneralFileInfo::getAttribute() const
{
  return _Mode;
}

bool GeneralFileInfo::copy(ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& bStop)
{
  QString Source = filePath();
  up:
  QFile FSource (Source);
  QFile FTarget (Target);
  QString Message;
  if (Source == Target)
  {

     int clicked = QMessageBox::information( NULL, i18n("KNC Information"), Message.sprintf(i18n("You cannot copy a file to itself.")),
                                          QMessageBox::Ok|QMessageBox::Default);
     if (clicked == QMessageBox::Ok)  return false;
     if (clicked == QMessageBox::Abort) {bStop = true; goto end;}
  }

  if (!FSource.open(IO_ReadOnly))
  {
     int clicked = QMessageBox::critical( NULL, i18n("KNC Problem"), Message.sprintf(i18n("Unable to open File %s."), Source.data()),
                                          QMessageBox::Retry|QMessageBox::Default, QMessageBox::Abort);
     if (clicked == QMessageBox::Abort){bStop = true; goto end;}
     if (clicked == QMessageBox::Retry) goto up;
  }
  if ((FTarget.exists()) && (!bOverwrite))
  {
    QFileInfo FS(FSource);
    QFileInfo FT(FTarget);
    QString Size1 = QString().setNum(FT.size());
    QString Size2 = QString().setNum(FS.size());
    QMessageBox mb( "KNC Warning", Message.sprintf(i18n("File %s exists:\nSize: %s\nDate: %s\n\nOverwrite with file?:\nSize: %s\nDate: %s"), Target.data(),
                     Size1.data(), FT.lastModified().toString().data(), Size2.data(), QFileInfo(FSource).lastModified().toString().data()),
                     QMessageBox::Warning,QMessageBox::Yes | QMessageBox::Default, QMessageBox::No, QMessageBox::Retry );
     mb.setButtonText( QMessageBox::Retry, "All" );
     switch( mb.exec() ) {case QMessageBox::No: return false; break;
       case QMessageBox::Retry: bOverwrite = true; break;}
  }
  if (!FTarget.open(IO_WriteOnly))
  {

    int clicked = QMessageBox::critical( NULL, i18n("KNC Problem"), Message.sprintf(i18n("Unable to open File %s."), Target.data()),
                                         QMessageBox::Retry|QMessageBox::Default, QMessageBox::Abort);
    if (clicked == QMessageBox::Abort) {bStop = true; goto end;}
    if (clicked == QMessageBox::Retry) goto up;
  }
  char block [BEST_BUFFERSIZE];
  while(1)
  {
    int read = FSource.readBlock(block, BEST_BUFFERSIZE);
    FTarget.writeBlock(block, read);
    if (PDialog) PDialog->addSize(read);
    if (read < BEST_BUFFERSIZE) break;
  }
  end:
  FSource.close();
  FTarget.close();
  return true;
}

bool GeneralFileInfo::rename(ProgressDialog* PFile, QString Target, bool& bOverwrite, bool& stop)
{
 QString Message;
 QString Source = filePath();
 up:
 if (Source.upper() == Target.upper())
 {
   char* TmpName;
   TmpName = tempnam(dirPath(), NULL);
   if (::rename(Source, TmpName) != 0) goto error;
   Source = TmpName;
 }
 if (::rename (Source, Target) != 0) goto error;
 return true;
 error:
   int clicked = QMessageBox::critical( NULL, i18n("KNC Problem"), Message.sprintf(i18n("Unable to rename %s."), Source.data()),
                                        QMessageBox::Ok|QMessageBox::Default, QMessageBox::Retry, QMessageBox::Abort);
   if (clicked == QMessageBox::Ok) return false;
   if (clicked == QMessageBox::Retry) goto up;
   if (clicked == QMessageBox::Abort) {stop = true;return false;}
}

bool GeneralFileInfo::remove(DeleteDialog* DDialog, bool& stop)
{
 QString Message;
 up:
 if (::remove(filePath()))
 {
   int clicked = QMessageBox::critical( NULL, i18n("KNC Problem"), Message.sprintf(i18n("Unable to remove %s."), filePath().data()),
                                       QMessageBox::Ok|QMessageBox::Default, QMessageBox::Retry, QMessageBox::Abort);

   if (clicked == QMessageBox::Ok) return false;
   if (clicked == QMessageBox::Retry) goto up;
   if (clicked == QMessageBox::Abort) {stop = true;return false;}
 }
 else
 {
   if (DDialog) DDialog->addSize(size());
   return true;
 }
 return true;
}

QPixmap GeneralDirInfo::getIcon()
{
  return kapp->getIconLoader()->loadMiniIcon("folder.xpm");
}

bool GeneralDirInfo::copy(ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& stop)
{
 QString Message;
 up:
  QDir Dir(Target);
  QString Name = Dir.dirName();
  Dir.cdUp();
  if (!Dir.exists()) goto error;
  if (!Dir.mkdir(Name)) goto error;
  return true;
  error:
   int clicked = QMessageBox::critical( NULL, i18n("KNC Problem"), Message.sprintf(i18n("Unable to create Directory %s."), filePath().data()),
                                       QMessageBox::Ok, QMessageBox::Retry, QMessageBox::Abort);

   if (clicked == QMessageBox::Retry) goto up;
   if (clicked == QMessageBox::Abort) {stop = true;return false;}
}

QPixmap GeneralLinkInfo::getIcon()
{
  QPixmap Icon;
  QPixmap Link = kapp->getIconLoader()->loadMiniIcon("link.xpm");
  if (!_bFileLink)
    Icon = kapp->getIconLoader()->loadMiniIcon("folder.xpm");
  else
    Icon = GeneralFileInfo::getIcon();

  //this is a very ugly workaround -> it is not possible to make a deep copy of a pixmap!
  QImage Image = Icon.convertToImage();
  Icon.convertFromImage(Image);

  //add space for the Link in the mask of Icon
  QPainter P(Icon.mask());
  P.fillRect(0,8,8,8,color1);
  P.end();
  //now we can copy the Link into Icon
  bitBlt(&Icon, 0,8, &Link, 0,8,8,8, CopyROP, true);

  return Icon;
}


bool GeneralLinkInfo::copy(ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& stop)
{
  QString Message;
  up:
  if (symlink(_Link, Target))
  {
   int clicked = QMessageBox::critical( NULL, i18n("KNC Problem"), Message.sprintf(i18n("Unable to copy Link %s."), filePath().data()),
                                       QMessageBox::Ok, QMessageBox::Retry, QMessageBox::Abort);
   if (clicked == QMessageBox::Retry) goto up;
   if (clicked == QMessageBox::Abort) {stop = true;return false;}
  }
  if (PDialog) PDialog->addSize(size());
  return true;
}

QPixmap GeneralDeviceInfo::getIcon()
{
  QString Icon;
  if (S_ISCHR(_Mode)) Icon = "chardevice.xpm";
  else Icon = "blockdevice.xpm";
  return kapp->getIconLoader()->loadMiniIcon(Icon);
}

bool GeneralDeviceInfo::copy(ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& stop)
{
  QString Message;
  QMessageBox::information( NULL, i18n("KNC Information"), Message.sprintf(i18n("KNC does not copy the device file %s"), filePath().data()),
                                       QMessageBox::Ok | QMessageBox::Default);
  return false;
}
bool GeneralDeviceInfo::rename(ProgressDialog* PFile, QString Target, bool& stop)
{
  QString Message;
  QMessageBox::information( NULL, i18n("KNC Information"), Message.sprintf(i18n("KNC does not rename the device file %s"), filePath().data()),
                                       QMessageBox::Ok | QMessageBox::Default);
  return false;
}
bool GeneralDeviceInfo::remove(DeleteDialog* PFile, bool& stop)
{
  QString Message;
  QMessageBox::information( NULL, i18n("KNC Information"), Message.sprintf(i18n("KNC does not remove the device file %s"), filePath().data()),
                                       QMessageBox::Ok | QMessageBox::Default);
  return false;
}

bool GeneralOtherInfo::copy(ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& stop)
{
  QString Message;
  QMessageBox::information( NULL, i18n("KNC Information"), Message.sprintf(i18n("KNC does not copy the file %s"), filePath().data()),
                                       QMessageBox::Ok | QMessageBox::Default);
  return false;
}
bool GeneralOtherInfo::rename(ProgressDialog* PFile, QString Target, bool& stop)
{
  QString Message;
  QMessageBox::information( NULL, i18n("KNC Information"), Message.sprintf(i18n("KNC does not rename the file %s"), filePath().data()),
                                       QMessageBox::Ok | QMessageBox::Default);
  return false;
}
bool GeneralOtherInfo::remove(DeleteDialog* PFile, bool& stop)
{
  QString Message;
  QMessageBox::information( NULL, i18n("KNC Information"), Message.sprintf(i18n("KNC does not remove the file %s"), filePath().data()),
                                       QMessageBox::Ok | QMessageBox::Default);
  return false;
}


void GeneralFileInfo::setOwner(QString Owner)
{
 struct passwd* pw = getpwnam(Owner);
 if (chown(filePath().data(), pw->pw_uid, _GroupId) != -1)
   _UserId = pw->pw_uid;
}
void GeneralFileInfo::setGroup(QString Group)
{
 struct group* ge = getgrnam(Group);
 if (chown(filePath().data(), _UserId, ge->gr_gid) != -1)
   _GroupId = ge->gr_gid;
}


