/***************************************************************************
                          knc.cpp  -  description                              
                             -------------------                                         
    begin                : Sat Jul 31 18:15:05 MEST 1999
                                           
    copyright            : (C) 1999 by Henrik Stormer                         
    email                : stormer@ifi.unizh.ch                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include <qstrlist.h>
#include <qlistview.h>
#include <qfileinfo.h>
#include <qmessagebox.h>
#include <qregexp.h>
#include <qlineedit.h>
#include <qmultilinedit.h>

#include <kiconloader.h>
#include <kfiledialog.h>
#include <knewpanner.h>
#include <kfontdialog.h>
#include <kkeydialog.h>
#include <knc.h>

#include "mylistview.h"
#include "resource.h"
#include "optionsdialog.h"
#include "pluginadmin.h"
#include <basepluginfactory.h>
#include "qlayout.h"
#include "mycommandline.h"
#include "listboxdialog.h"
#include "mycolordialog.h"

KNCApp::KNCApp()
{
	_config=kapp->getConfig();
  initView();
  initMenuBar();
  initToolBar();
  initStatusBar();
  initKeyAccel();
  changeMenuAccels();
	readOptions();
  //_bookmarks = new QString[64];
}

KNCApp::~KNCApp()
{
  delete PluginAdmin::_instance;
  //delete []_bookmarks;
}

void KNCApp::initKeyAccel()
{
	key_accel = new KAccel( this );

  key_accel->insertItem (i18n("Change Windows"), "Change Windows", Key_Tab);
  key_accel->connectItem ("Change Windows", this, SLOT (slotChangeWindows() ) );

  key_accel->insertItem (i18n("Up"), "Up", Key_Up);
  key_accel->connectItem ("Up", this, SLOT (slotUp() ) );

  key_accel->insertItem (i18n("Down"), "Down", Key_Down);
  key_accel->connectItem ("Down", this, SLOT (slotDown() ) );

  key_accel->insertItem (i18n("Left"), "Left", Key_Left);
  key_accel->connectItem ("Left", this, SLOT (slotLeft() ) );

  key_accel->insertItem (i18n("Right"), "Right", Key_Right);
  key_accel->connectItem ("Right", this, SLOT (slotRight() ) );

  key_accel->insertItem (i18n("Page Up"), "Page Up", Key_PageUp);
  key_accel->connectItem ("Page Up", this, SLOT (slotPageUp() ) );

  key_accel->insertItem (i18n("Page Down"), "Page Down", Key_PageDown);
  key_accel->connectItem ("Page Down", this, SLOT (slotPageDown() ) );

  key_accel->insertItem (i18n("Scroll Begin"), "Scroll Begin", Key_Home);
  key_accel->connectItem ("Scroll Begin", this, SLOT (slotScrollBegin() ) );

  key_accel->insertItem (i18n("Scroll End"), "Scroll End", Key_End);
  key_accel->connectItem ("Scroll End", this, SLOT (slotScrollEnd() ) );

  key_accel->insertItem (i18n("Invert Selection"), "Invert Selection", Key_Space);
  key_accel->connectItem ("Invert Selection", this, SLOT (slotInvertSelection() ) );

  key_accel->insertItem (i18n("Directory Up"), "Directory Up", Key_Backspace);
  key_accel->connectItem ("Directory Up", this, SLOT (slotDirectoryUp() ) );

  key_accel->insertItem (i18n("Enter"), "Enter", Key_Return);
  key_accel->connectItem ("Enter", this, SLOT (slotEnter() ) );

  key_accel->insertItem (i18n("Execute with"), "Execute with", Key_Return + SHIFT);
  key_accel->connectItem ("Execute with", this, SLOT (slotCommandExecuteWith() ) );

  key_accel->insertItem (i18n("Reset Substring"), "Reset Substring", Key_Escape);
  key_accel->connectItem ("Reset Substring", this, SLOT (slotResetSubString() ) );

  //this item appear in the menu bar, so the connection moved to change menu accels
  key_accel->insertItem (i18n("Refresh Left"), "Refresh Left", CTRL + Key_L);
  key_accel->insertItem (i18n("Refresh Right"), "Refresh Right", CTRL + Key_R);

  key_accel->insertItem (i18n("Selection"), "Selection", Key_Plus);
  key_accel->insertItem (i18n("Unselection"), "Unselection", Key_Minus);
  key_accel->insertItem (i18n("View"), "View", Key_F3);
  key_accel->insertItem (i18n("Edit"), "Edit", Key_F4);
  key_accel->insertItem (i18n("Copy"), "Copy", Key_F5);
  key_accel->insertItem (i18n("Move"), "Move", Key_F6);
  key_accel->insertItem (i18n("NewDir"), "NewDir", Key_F7);
  key_accel->insertItem (i18n("Delete"), "Delete", Key_F8);
  key_accel->insertItem (i18n("Open Terminal"), "Open Terminal", CTRL + Key_T);
  key_accel->insertItem (i18n("Open Find Utility"), "Open Find Utility", CTRL + Key_F);
  key_accel->insertItem (i18n("Change Attributes"), "Change Attributes", CTRL + Key_A);
  key_accel->insertItem (i18n("Invoke Commandline"), "Invoke Commandline", CTRL + Key_L);
  key_accel->insertItem (i18n("Add Bookmark"), "Add Bookmark", CTRL + Key_B);
  key_accel->insertItem (i18n("Remove Bookmark"), "Remove Bookmark", CTRL + SHIFT + Key_B);

  PluginFactoryIterator it = PluginAdmin::instance().getPluginFactoryIterator();
  for (;it.current();++it)
    it.current()->initKeyAccel(key_accel, this);


	key_accel->readSettings();	
}

void KNCApp::changeMenuAccels()
{
  key_accel->disconnectItem ("Refresh Left", this, SLOT (slotLeftRefresh() ) );
  key_accel->connectItem ("Refresh Left", this, SLOT (slotLeftRefresh() ) );

  key_accel->disconnectItem ("Refresh Right", this, SLOT (slotRightRefresh() ) );
  key_accel->connectItem ("Refresh Right", this, SLOT (slotRightRefresh() ) );

  key_accel->disconnectItem ("Selection", this, SLOT (slotCommandSelect() ) );
  key_accel->connectItem ("Selection", this, SLOT (slotCommandSelect() ) );

  key_accel->disconnectItem ("Unselection", this, SLOT (slotCommandUnselect() ) );
  key_accel->connectItem ("Unselection", this, SLOT (slotCommandUnselect() ) );

  key_accel->disconnectItem ("View", this, SLOT (slotCommandView() ) );
  key_accel->connectItem ("View", this, SLOT (slotCommandView() ) );

  key_accel->disconnectItem ("Edit", this, SLOT (slotCommandEdit() ) );
  key_accel->connectItem ("Edit", this, SLOT (slotCommandEdit() ) );

  key_accel->disconnectItem ("Copy", this, SLOT (slotCommandCopy() ) );
  key_accel->connectItem ("Copy", this, SLOT (slotCommandCopy() ) );

  key_accel->disconnectItem ("Move", this, SLOT (slotCommandMove() ) );
  key_accel->connectItem ("Move", this, SLOT (slotCommandMove() ) );

  key_accel->disconnectItem ("NewDir", this, SLOT (slotCommandNewDir() ) );
  key_accel->connectItem ("NewDir", this, SLOT (slotCommandNewDir() ) );

  key_accel->disconnectItem ("Delete", this, SLOT (slotCommandDelete() ) );
  key_accel->connectItem ("Delete", this, SLOT (slotCommandDelete() ) );

  key_accel->disconnectItem ("Open Terminal", this, SLOT (slotCommandOpenTerminal() ) );
  key_accel->connectItem ("Open Terminal", this, SLOT (slotCommandOpenTerminal() ) );

  key_accel->disconnectItem ("Open Find Utility", this, SLOT (slotCommandOpenFindUtility() ) );
  key_accel->connectItem ("Open Find Utility", this, SLOT (slotCommandOpenFindUtility() ) );

  key_accel->disconnectItem ("Change Attributes", this, SLOT (slotCommandChangeAttributes() ) );
  key_accel->connectItem ("Change Attributes", this, SLOT (slotCommandChangeAttributes() ) );

  key_accel->disconnectItem ("Invoke Commandline", this, SLOT (slotCommandInvokeCommandLine() ) );
  key_accel->connectItem ("Invoke Commandline", this, SLOT (slotCommandInvokeCommandLine() ) );

  key_accel->disconnectItem ("Add Bookmark", this, SLOT (slotBookmarkAdd() ) );
  key_accel->connectItem ("Add Bookmark", this, SLOT (slotBookmarkAdd() ) );

  key_accel->disconnectItem ("Remove Bookmark", this, SLOT (slotBookmarkRemove() ) );
  key_accel->connectItem ("Remove Bookmark", this, SLOT (slotBookmarkRemove() ) );

  key_accel->changeMenuAccel(_left_menu,ID_LEFT_REFRESH ,"Refresh Left" );
  key_accel->changeMenuAccel(_right_menu,ID_RIGHT_REFRESH ,"Refresh Right" );
  key_accel->changeMenuAccel(_command_menu,ID_COMMAND_SELECT ,"Selection" );
  key_accel->changeMenuAccel(_command_menu,ID_COMMAND_UNSELECT ,"Unselection" );
  key_accel->changeMenuAccel(_command_menu,ID_COMMAND_VIEW ,"View" );
  key_accel->changeMenuAccel(_command_menu,ID_COMMAND_EDIT,"Edit" );
  key_accel->changeMenuAccel(_command_menu,ID_COMMAND_COPY ,"Copy" );
  key_accel->changeMenuAccel(_command_menu,ID_COMMAND_MOVE ,"Move" );
  key_accel->changeMenuAccel(_command_menu,ID_COMMAND_NEWDIR,"NewDir" );
  key_accel->changeMenuAccel(_command_menu,ID_COMMAND_DELETE ,"Delete" );
  key_accel->changeMenuAccel(_command_menu,ID_COMMAND_CHANGEATTRIB ,"Change Attributes" );
  key_accel->changeMenuAccel(_command_menu,ID_COMMAND_OPENTERMINAL ,"Open Terminal" );
  key_accel->changeMenuAccel(_command_menu,ID_COMMAND_OPENFINDUTILITY ,"Open Find Utility" );
  key_accel->changeMenuAccel(_command_menu,ID_COMMAND_INVOKECOMMANDLINE ,"Invoke Commandline" );
  key_accel->changeMenuAccel(_bookmark_menu,ID_BOOKMARK_ADD ,"Add Bookmark" );
  key_accel->changeMenuAccel(_bookmark_menu,ID_BOOKMARK_REMOVE ,"Remove Bookmark" );

  PluginFactoryIterator it = PluginAdmin::instance().getPluginFactoryIterator();
  for (;it.current();++it)
    it.current()->changeMenuAccel(key_accel, _command_menu);

}

void KNCApp::initMenuBar()
{
  _left_menu = new QPopupMenu();
  _left_menu->insertItem(i18n("Show &Hidden Files"),  ID_LEFT_HIDDEN);
  _left_menu->insertSeparator();
  _left_menu->insertItem(i18n("Sort by &Name"), ID_LEFT_SORTNAME);
  _left_menu->insertItem(i18n("Sort by &Size"), ID_LEFT_SORTSIZE);
  _left_menu->insertItem(i18n("Sort by &Attributes"), ID_LEFT_SORTATTRIB);
  _left_menu->insertItem(i18n("Sort by &Owner"), ID_LEFT_SORTOWNER);
  _left_menu->insertItem(i18n("Sort by &Group"), ID_LEFT_SORTGROUP);
  _left_menu->insertItem(i18n("Sort by &Last Modified Time"), ID_LEFT_SORTLASTMODTIME);
  _left_menu->insertItem(i18n("Sort by &Last Read Time"), ID_LEFT_SORTLASTREADTIME);
  _left_menu->insertSeparator();
  _left_menu->insertItem(i18n("&Refresh"), ID_LEFT_REFRESH);

  _command_menu = new QPopupMenu();
  _command_menu->insertItem(i18n("&View File"), ID_COMMAND_VIEW );
  _command_menu->insertItem(i18n("&Edit File"), ID_COMMAND_EDIT );
  _command_menu->insertItem(i18n("&Copy Files"), ID_COMMAND_COPY );
  _command_menu->insertItem(i18n("&Move/Rename Files"), ID_COMMAND_MOVE );
  _command_menu->insertItem(i18n("C&reate Directory"), ID_COMMAND_NEWDIR );
  _command_menu->insertItem(i18n("&Delete Files"), ID_COMMAND_DELETE );
  _command_menu->insertItem(i18n("Change File &Attributes"), ID_COMMAND_CHANGEATTRIB );
  _command_menu->insertSeparator();
  _command_menu->insertItem(i18n("&Select Files"), ID_COMMAND_SELECT );
  _command_menu->insertItem(i18n("&Unselect Files"), ID_COMMAND_UNSELECT );
  _command_menu->insertSeparator();
  _command_menu->insertItem(i18n("&Open Terminal"), ID_COMMAND_OPENTERMINAL );
  _command_menu->insertItem(i18n("&Open Find Utility"), ID_COMMAND_OPENFINDUTILITY );
//  _command_menu->insertItem(i18n("&Open User Menu"), ID_COMMAND_OPENUSERMENU );
  _command_menu->insertSeparator();
  _command_menu->insertItem(i18n("&Invoke CommandLine"), ID_COMMAND_INVOKECOMMANDLINE );

  //now let the Plugins create their own menus
  PluginFactoryIterator it = PluginAdmin::instance().getPluginFactoryIterator();
  for (;it.current();++it)
    if (it.current()->addMenuItems(_command_menu))
      _command_menu->insertSeparator();

  _command_menu->insertItem(i18n("E&xit"), ID_COMMAND_QUIT );

  _view_menu = new QPopupMenu();
  _view_menu->insertItem(i18n("&Status Bar"), ID_VIEW_STATUSBAR);
  _view_menu->insertItem(i18n("&Toolbar"), ID_VIEW_TOOLBAR);
  _view_menu->insertItem(i18n("&Directory Line"), ID_VIEW_DIRECTORYLINE);
  _view_menu->insertItem(i18n("&Command Line"), ID_VIEW_COMMANDLINE);

  _bookmark_menu = new QPopupMenu();
  _bookmark_menu->insertItem(i18n("&Add Bookmark"), ID_BOOKMARK_ADD);
  _bookmark_menu->insertItem(i18n("&Remove Bookmark"), ID_BOOKMARK_REMOVE);
  _bookmark_menu->insertSeparator();

  _options_menu = new QPopupMenu();
  _options_menu->insertItem(i18n("&Options"), ID_OPTIONS_GENERAL);
  _options_menu->insertItem(i18n("&Configure Keys"), ID_OPTIONS_KEY);
  _options_menu->insertItem(i18n("Set &Font"), ID_OPTIONS_FONT);
  _options_menu->insertSeparator();
  _options_menu->insertItem(i18n("&Colors"), ID_OPTIONS_COLORS);

  _right_menu = new QPopupMenu();
  _right_menu->insertItem(i18n("Show &Hidden Files"),  ID_RIGHT_HIDDEN);
  _right_menu->insertSeparator();
  _right_menu->insertItem(i18n("Sort by &Name"), ID_RIGHT_SORTNAME);
  _right_menu->insertItem(i18n("Sort by &Size"), ID_RIGHT_SORTSIZE);
  _right_menu->insertItem(i18n("Sort by &Attributes"), ID_RIGHT_SORTATTRIB);
  _right_menu->insertItem(i18n("Sort by &Owner"), ID_RIGHT_SORTOWNER);
  _right_menu->insertItem(i18n("Sort by &Group"), ID_RIGHT_SORTGROUP);
  _right_menu->insertItem(i18n("Sort by &Last Modified Time"), ID_RIGHT_SORTLASTMODTIME);
  _right_menu->insertItem(i18n("Sort by &Last Read Time"), ID_RIGHT_SORTLASTREADTIME);
  _right_menu->insertSeparator();
  _right_menu->insertItem(i18n("&Refresh"), ID_RIGHT_REFRESH);


  QString aboutstring=kapp->appName()+" Version: "+VERSION+
     QString("\n\n") +
     QString("(c) 1999/2000 Henrik Stormer\n") +
     QString("\n\n") +
     QString("Send bugreports, errors, and suggestions to:\n") +
     QString("stormer@ifi.unizh.ch\n");

  _help_menu = new QPopupMenu();
  _help_menu = kapp->getHelpMenu(true, aboutstring );

  menuBar()->insertItem(i18n("&Left"), _left_menu);
  menuBar()->insertItem(i18n("&Command"), _command_menu);
  menuBar()->insertItem(i18n("&View"), _view_menu);
  menuBar()->insertItem(i18n("&Bookmarks"), _bookmark_menu);
  menuBar()->insertItem(i18n("&Options"), _options_menu);
  menuBar()->insertItem(i18n("&Help"), _help_menu);
  menuBar()->insertSeparator();
  menuBar()->insertItem(i18n("&Right"), _right_menu);


  connect(_left_menu,SIGNAL(activated(int)),SLOT(commandCallback(int)));
  connect(_left_menu,SIGNAL(highlighted(int)),SLOT(statusCallback(int)));

  connect(_command_menu,SIGNAL(activated(int)),SLOT(commandCallback(int)));
  connect(_command_menu,SIGNAL(highlighted(int)),SLOT(statusCallback(int)));

  connect(_view_menu,SIGNAL(activated(int)),SLOT(commandCallback(int)));
  connect(_view_menu,SIGNAL(highlighted(int)),SLOT(statusCallback(int)));

  connect(_bookmark_menu,SIGNAL(activated(int)),SLOT(commandCallback(int)));
  connect(_bookmark_menu,SIGNAL(highlighted(int)),SLOT(statusCallback(int)));

  connect(_options_menu,SIGNAL(activated(int)),SLOT(commandCallback(int)));
  connect(_options_menu,SIGNAL(highlighted(int)),SLOT(statusCallback(int)));

  connect(_right_menu,SIGNAL(activated(int)),SLOT(commandCallback(int)));
  connect(_right_menu,SIGNAL(highlighted(int)),SLOT(statusCallback(int)));
}



void KNCApp::initToolBar()
{
  toolBar()->insertButton(Icon("view.xpm"), ID_COMMAND_VIEW, true, i18n("View File") );
  toolBar()->insertButton(Icon("edit.xpm"), ID_COMMAND_EDIT, true, i18n("Edit File") );
  toolBar()->insertButton(Icon("copy.xpm"), ID_COMMAND_COPY, true, i18n("Copy Files") );
  toolBar()->insertButton(Icon("move.xpm"), ID_COMMAND_COPY, true, i18n("Move Files") );
  toolBar()->insertButton(Icon("newdir.xpm"), ID_COMMAND_NEWDIR, true, i18n("Create New Directory") );
  toolBar()->insertButton(Icon("delete.xpm"), ID_COMMAND_DELETE, true, i18n("Delete Files") );
  toolBar()->insertButton(Icon("changeattrib.xpm"), ID_COMMAND_CHANGEATTRIB, true, i18n("Change Attributes") );
  toolBar()->insertSeparator();
  toolBar()->insertButton(Icon("find.xpm"), ID_COMMAND_OPENFINDUTILITY, true, i18n("Search for Files") );
  toolBar()->insertButton(Icon("terminal.xpm"), ID_COMMAND_OPENTERMINAL, true, i18n("Open Terminal") );
  toolBar()->insertSeparator();

  //now let the Plugins create their own toolbarButtons
  PluginFactoryIterator it = PluginAdmin::instance().getPluginFactoryIterator();
  for (;it.current();++it)
    if (it.current()->addToolbarItems(toolBar()))
      _command_menu->insertSeparator();

  toolBar()->insertButton(Icon("quit.xpm"), ID_COMMAND_QUIT, true, i18n("Exit KNC") );
  toolBar()->insertSeparator();

  connect(toolBar(), SIGNAL(clicked(int)), SLOT(commandCallback(int)));
	// connect for the status help on holing icons pressed with the mouse button
  connect(toolBar(), SIGNAL(pressed(int)), SLOT(statusCallback(int)));
}

class MyStatusBar : public KStatusBar
{
 public:
  MyStatusBar(QWidget* parent, const char* name=0) : KStatusBar(parent, name)
  {
    _LeftSide = new QLabel(this);
//    _LeftSide->setText("");
  	_LeftSide->setFrameStyle(QFrame::Panel | QFrame::Sunken);
    _LeftSide->setLineWidth(1);
    insertWidget(_LeftSide, 200, 100);

    _RightSide = new QLabel(this);
//    _RightSide->setText("");
  	_RightSide->setFrameStyle(QFrame::Panel | QFrame::Sunken);
    _RightSide->setLineWidth(1);
    insertWidget(_RightSide, 200, 101);

 //   initStatusBar();
  }

  void setLeftSide(QString Message) {_LeftSide->setText(Message);}
  void setRightSide(QString Message) {_RightSide->setText(Message);}

 protected:

   virtual void resizeEvent ( QResizeEvent * e)
  {
    _LeftSide->resize(e->size().width()/2 ,20);
    _RightSide->resize(e->size().width()/2, 20);
    updateRects(false);
  }
 private:
 QLabel* _LeftSide;
 QLabel* _RightSide;
};

void KNCApp::initStatusBar()
{
  setStatusBar(new MyStatusBar(this));
}


void KNCApp::enableCommand(int id_)
{
  menuBar()->setItemEnabled(id_,true);
  toolBar()->setItemEnabled(id_,true);
}

void KNCApp::disableCommand(int id_)
{
  menuBar()->setItemEnabled(id_,false);
  toolBar()->setItemEnabled(id_,false);
}

void KNCApp::saveOptions()
{	
	_config->setGroup("General Options");
	_config->writeEntry("Geometry", size() );
  _config->writeEntry("View Toolbar", toolBar()->isVisible());
  _config->writeEntry("View Statusbar",statusBar()->isVisible());
  _config->writeEntry("View Directory Line", _bViewDirectoryLine);
  _config->writeEntry("View Command Line", _bViewCommandLine);
  int sepPos;
  if (_bViewCommandLine) sepPos = _CommandPanner->separatorPos(); else sepPos = _CommandPannerPos;
  _config->writeEntry("Command Line Pos", sepPos);

  _config->writeEntry("MenuBarPos", (int)menuBar()->menuBarPos());
  _config->writeEntry("ToolBarPos",  (int)toolBar()->barPos());
	_config->writeEntry("Font", _LeftBox->font());
	_config->writeEntry("Foreground Color", _LeftBox->palette().normal().text());
	_config->writeEntry("Background Color", _LeftBox->palette().normal().base());
	_config->writeEntry("Focus Color", _LeftBox->getFocusColor());
	_config->writeEntry("Focus Cursor Color", _LeftBox->getFocusCursorColor());

  _config->writeEntry("Show Delete Message Box", _bShowDeleteMessageBox);
  _config->writeEntry("Execute Kfm", _bExecKfm);
  _config->writeEntry("Recurse Subdirs", _LeftBox->getSelectionIncludeSubdirs());

  _config->setGroup("Left View");
  _config->writeEntry("Show Hidden Files", _LeftBox->getShowHiddenFiles());
  _config->writeEntry("Directory", _LeftBox->getDir());
  _config->setGroup("Right View");
  _config->writeEntry("Show Hidden Files", _RightBox->getShowHiddenFiles());
  _config->writeEntry("Directory", _RightBox->getDir());

  _config->setGroup("Column Width");
  _config->writeEntry("lName", _LeftBox->columnWidth(0));
  _config->writeEntry("lSize", _LeftBox->columnWidth(1));
  _config->writeEntry("lAttrib", _LeftBox->columnWidth(2));
  _config->writeEntry("lOwner", _LeftBox->columnWidth(3));
  _config->writeEntry("lGroup", _LeftBox->columnWidth(4));
  _config->writeEntry("lLastMod", _LeftBox->columnWidth(5));
  _config->writeEntry("lLastRead", _LeftBox->columnWidth(6));
  _config->writeEntry("rName", _RightBox->columnWidth(0));
  _config->writeEntry("rSize", _RightBox->columnWidth(1));
  _config->writeEntry("rAttrib", _RightBox->columnWidth(2));
  _config->writeEntry("lOwner", _RightBox->columnWidth(3));
  _config->writeEntry("lGroup", _RightBox->columnWidth(4));
  _config->writeEntry("rLastMod", _RightBox->columnWidth(5));
  _config->writeEntry("rLastRead", _RightBox->columnWidth(6));

  _config->setGroup("External Program Calls");
  _config->writeEntry("Viewer", _Viewer);
  _config->writeEntry("Editor", _Editor);
  _config->writeEntry("Find Utility", _Find);

  _config->setGroup("Bookmarks");
  int count = 0;
  for (int i=0;i<_nBookmarks;i++)
  {
    if (_bookmarks[i] != "")
    {
      count++;
      _config->writeEntry(QString ("Bookmark ") + QString().setNum(count), _bookmarks[i]);
    }
  }
  _config->writeEntry("Num Bookmarks", count);

}

void KNCApp::readOptions()
{
	
	_config->setGroup("General Options");
  _bViewDirectoryLine = _config->readBoolEntry("View Directory Line", true);
  _view_menu->setItemChecked(ID_VIEW_DIRECTORYLINE, _bViewDirectoryLine);
  _Panner->showLabels(_bViewDirectoryLine);

  _bViewCommandLine = _config->readBoolEntry("View Command Line", false);
  _view_menu->setItemChecked(ID_VIEW_COMMANDLINE, _bViewCommandLine);
  _CommandPannerPos = _config->readNumEntry("Command Line Pos", 80);
  slotViewCommandLine(false); //no toggle!

  _bViewStatusBar = _config->readBoolEntry("View Statusbar", true);
	_view_menu->setItemChecked(ID_VIEW_STATUSBAR, _bViewStatusBar);
	if(!_bViewStatusBar) enableStatusBar(KStatusBar::Hide);

	_bViewToolbar = _config->readBoolEntry("View Toolbar", true);
	_view_menu->setItemChecked(ID_VIEW_TOOLBAR, _bViewToolbar);
	if(!_bViewToolbar)	enableToolBar(KToolBar::Hide);

  QPalette palette = _LeftBox->palette();
  QColorGroup C = palette.normal();

  QColor FColor, BColor, FocusColor, FocusCursorColor;
  QColor TempColor(255,255,195);

  FColor = _config->readColorEntry("Foreground Color", &black);
  BColor = _config->readColorEntry("Background Color", &TempColor);
  FocusColor = _config->readColorEntry("Focus Color", &black);
  FocusCursorColor = _config->readColorEntry("Focus Cursor Color", &black);

  palette.setNormal(QColorGroup(C.foreground(), C.background(), C.light(), C.dark(), C.mid(), FColor, BColor));
  palette.setActive(QColorGroup(C.foreground(), C.background(), C.light(), C.dark(), C.mid(), FColor, BColor));

  _LeftBox->setPalette(palette);
  _LeftBox->setFocusColor(FocusColor);
  _LeftBox->setFocusCursorColor(FocusCursorColor);
  _RightBox->setPalette(palette);
  _RightBox->setFocusColor(FocusColor);
  _RightBox->setFocusCursorColor(FocusCursorColor);


  QSize size(1000,500);
  size =_config->readSizeEntry("Geometry",&size);
	if(!size.isEmpty())
		resize(size);
  _bShowDeleteMessageBox = _config->readBoolEntry("Show Delete Message Box", true);
  _bExecKfm = _config->readBoolEntry("Execute Kfm", true);
  bool Sel = _config->readBoolEntry("Recurse Subdirs", true);
  _LeftBox->setSelectionIncludeSubdirs(Sel);
  _RightBox->setSelectionIncludeSubdirs(Sel);

  _config->setGroup("External Program Calls");
  _Viewer = _config->readEntry("Viewer", "kless");
  _Editor = _config->readEntry("Editor", "kwrite");
  _Find = _config->readEntry("Find Utility", "kfind");

  _config->setGroup("Left View");
  bool sh = _config->readBoolEntry("Show Hidden Files", true);
	_left_menu->setItemChecked(ID_LEFT_HIDDEN, sh);
  _LeftBox->setShowHiddenFiles(sh);
  QString Path = _config->readEntry("Directory","/");
  if (Path.right(1) != "/") Path += "/";
  _LeftBox->setDirectory(Path);

  _config->setGroup("Right View");
  sh = _config->readBoolEntry("Show Hidden Files", true);
	_right_menu->setItemChecked(ID_RIGHT_HIDDEN, sh);
  _RightBox->setShowHiddenFiles(sh);
  Path = _config->readEntry("Directory","/");
  if (Path.right(1) != "/") Path += "/";
  _RightBox->setDirectory(Path);
  slotDirectoryChanged("");

  _config->setGroup("Bookmarks");
  _nBookmarks = _config->readNumEntry("Num Bookmarks",0);

  for (int i=0;i<_nBookmarks;i++)
  {
    _bookmarks[i] = _config->readEntry(QString("Bookmark ") + QString().setNum(i+1),"xxx");
    _bookmark_menu->insertItem(_bookmarks[i], i+ID_BOOKMARK_MIN);
  }
}

bool KNCApp::queryExit()
{
	saveOptions();
	return true;
}

void KNCApp::slotStatusMsg(const char *text)
{
  statusBar()->clear();
  statusBar()->changeItem(text, ID_STATUS_MSG );
}


void KNCApp::slotStatusHelpMsg(const char *text)
{
  statusBar()->message(text, 2000);
}



void KNCApp::slotLeftSortName() {_LeftBox->setSorting(0,true);}
void KNCApp::slotLeftSortSize() {_LeftBox->setSorting(1,true);}
void KNCApp::slotLeftSortAttrib() {_LeftBox->setSorting(2,true);}
void KNCApp::slotLeftSortOwner() {_LeftBox->setSorting(3,true);}
void KNCApp::slotLeftSortGroup() {_LeftBox->setSorting(4,true);}
void KNCApp::slotLeftSortLastModTime() {_LeftBox->setSorting(5,true);}
void KNCApp::slotLeftSortLastReadTime() {_LeftBox->setSorting(6,true);}
void KNCApp::slotLeftRefresh() {_LeftBox->refresh();}
void KNCApp::slotRightSortName() {_RightBox->setSorting(0,true);}
void KNCApp::slotRightSortSize(){_RightBox->setSorting(1,true);}
void KNCApp::slotRightSortAttrib(){_RightBox->setSorting(2,true);}
void KNCApp::slotRightSortOwner() {_RightBox->setSorting(3,true);}
void KNCApp::slotRightSortGroup() {_RightBox->setSorting(4,true);}
void KNCApp::slotRightSortLastModTime(){_RightBox->setSorting(5,true);}
void KNCApp::slotRightSortLastReadTime(){_RightBox->setSorting(6,true);}
void KNCApp::slotRightRefresh(){_RightBox->refresh();}

void KNCApp::slotLeftHidden()
{
  bool sh = _LeftBox->getShowHiddenFiles();
 	_left_menu->setItemChecked(ID_LEFT_HIDDEN, !sh);
  _LeftBox->setShowHiddenFiles(!sh);
  slotStatusMsg(i18n("Ready."));
}

void KNCApp::slotRightHidden()
{
  bool sh = _RightBox->getShowHiddenFiles();
 	_right_menu->setItemChecked(ID_RIGHT_HIDDEN, !sh);
  _RightBox->setShowHiddenFiles(!sh);
  slotStatusMsg(i18n("Ready."));
}


void KNCApp::slotOptionsGeneral()
{
  OptionsDialog Dialog(this, i18n("Options"), _Viewer, _Editor, _Find, _bShowDeleteMessageBox,
                       _bExecKfm, _LeftBox->getSelectionIncludeSubdirs());
  if (Dialog.exec())
  {
    _Viewer = Dialog.getViewer();
    _Editor = Dialog.getEditor();
    _Find = Dialog.getFind();
    _bShowDeleteMessageBox = Dialog.getShowDeleteMessageBox();
    _bExecKfm = Dialog.getExecKfm();
    _LeftBox->setSelectionIncludeSubdirs(Dialog.getRecurseSubdirs());
    _RightBox->setSelectionIncludeSubdirs(Dialog.getRecurseSubdirs());
  }
}

void KNCApp::slotOptionsFont()
{
  KFontDialog Dialog(this);
  QFont Font;
  if (Dialog.getFont(Font))
  {
    _LeftBox->setFont(Font);
    _RightBox->setFont(Font);
  }
}

void KNCApp::slotOptionsColors()
{
 MyColorDialog Dialog;

 QColor ForegroundColor = _LeftBox->palette().normal().text();
 QColor BackgroundColor = _LeftBox->palette().normal().base();
 QColor FocusColor = _LeftBox->getFocusColor();
 QColor FocusCursorColor = _LeftBox->getFocusCursorColor();


 if (Dialog.getColor( ForegroundColor, BackgroundColor, FocusColor, FocusCursorColor ))
 {
    QPalette palette = _LeftBox->palette();
    QColorGroup C = palette.normal();
    palette.setNormal(QColorGroup(C.foreground(), C.background(), C.light(), C.dark(), C.mid(), ForegroundColor, BackgroundColor));
    palette.setActive(QColorGroup(C.foreground(), C.background(), C.light(), C.dark(), C.mid(), ForegroundColor, BackgroundColor));
    _LeftBox->setPalette( palette );
    _LeftBox->setFocusColor( FocusColor );
    _LeftBox->setFocusCursorColor( FocusCursorColor );
	  _RightBox->setPalette( palette );
    _RightBox->setFocusColor( FocusColor );
    _RightBox->setFocusCursorColor( FocusCursorColor );
  }
}


void KNCApp::slotViewToolBar()
{
  _bViewToolbar = !_bViewToolbar;
  _view_menu->setItemChecked(ID_VIEW_TOOLBAR, _bViewToolbar);
	if(!_bViewToolbar) enableToolBar(KToolBar::Hide);
    else enableToolBar(KToolBar::Show);
  slotStatusMsg(i18n("Ready."));
}

void KNCApp::slotViewStatusBar()
{
  _bViewStatusBar = !_bViewStatusBar;
 	_view_menu->setItemChecked(ID_VIEW_STATUSBAR, _bViewStatusBar);
	if(!_bViewStatusBar) enableStatusBar(KStatusBar::Hide);
  else enableStatusBar(KStatusBar::Show);
  slotStatusMsg(i18n("Ready."));
}

void KNCApp::slotViewDirectoryLine()
{
  _bViewDirectoryLine = !_bViewDirectoryLine;
 	_view_menu->setItemChecked(ID_VIEW_DIRECTORYLINE, _bViewDirectoryLine);
  _Panner->showLabels(_bViewDirectoryLine);
  slotStatusMsg(i18n("Ready."));
}

void KNCApp::slotViewCommandLine(bool toggle)
{
  if (toggle) _bViewCommandLine = !_bViewCommandLine;
 	_view_menu->setItemChecked(ID_VIEW_COMMANDLINE, _bViewCommandLine);
  if (_bViewCommandLine)
  {
    _CommandPanner->setSeparatorPos(_CommandPannerPos);
  }
  else
  {
    if (toggle) _CommandPannerPos = _CommandPanner->separatorPos();
    _CommandPanner->setSeparatorPos(100);
  }
  QRect rMainGeom= _CommandPanner->geometry();
  _CommandPanner->resize(rMainGeom.width()+1,rMainGeom.height());
  _CommandPanner->resize(rMainGeom.width(),rMainGeom.height());
  slotStatusMsg(i18n("Ready."));
}


void KNCApp::commandCallback(int id)
{
  //first let the Plugins look
  PluginFactoryIterator it = PluginAdmin::instance().getPluginFactoryIterator();
  for (;it.current();++it)
    if (it.current()->commandCallback(id)) return;

  switch (id){
   case ID_LEFT_HIDDEN:  slotLeftHidden(); break;
   case ID_LEFT_SORTNAME: slotLeftSortName(); break;
   case ID_LEFT_SORTSIZE: slotLeftSortSize(); break;
   case ID_LEFT_SORTATTRIB: slotLeftSortAttrib(); break;
   case ID_LEFT_SORTOWNER: slotLeftSortOwner(); break;
   case ID_LEFT_SORTGROUP: slotLeftSortGroup(); break;
   case ID_LEFT_SORTLASTMODTIME: slotLeftSortLastModTime(); break;
   case ID_LEFT_SORTLASTREADTIME: slotLeftSortLastReadTime(); break;
   case ID_LEFT_REFRESH:  slotLeftRefresh(); break;

   case ID_COMMAND_SELECT: slotCommandSelect(); break;
   case ID_COMMAND_UNSELECT: slotCommandUnselect(); break;
   case ID_COMMAND_VIEW: slotCommandView(); break;
   case ID_COMMAND_EDIT: slotCommandEdit(); break;
   case ID_COMMAND_COPY:  slotCommandCopy(); break;
   case ID_COMMAND_MOVE:   slotCommandMove(); break;
   case ID_COMMAND_NEWDIR: slotCommandNewDir(); break;
   case ID_COMMAND_DELETE: slotCommandDelete(); break;
   case ID_COMMAND_CHANGEATTRIB: slotCommandChangeAttributes(); break;
   case ID_COMMAND_OPENTERMINAL: slotCommandOpenTerminal(); break;
   case ID_COMMAND_OPENFINDUTILITY: slotCommandOpenFindUtility(); break;
   case ID_COMMAND_OPENUSERMENU: slotCommandOpenUserMenu(); break;
   case ID_COMMAND_INVOKECOMMANDLINE: slotCommandInvokeCommandLine();break;
   case ID_COMMAND_QUIT:  slotCommandQuit(); break;

   case ID_VIEW_TOOLBAR: slotViewToolBar(); break;
   case ID_VIEW_STATUSBAR: slotViewStatusBar(); break;
   case ID_VIEW_DIRECTORYLINE: slotViewDirectoryLine(); break;
   case ID_VIEW_COMMANDLINE: slotViewCommandLine(); break;

   case ID_BOOKMARK_ADD: slotBookmarkAdd(); break;
   case ID_BOOKMARK_REMOVE: slotBookmarkRemove(); break;
   case ID_OPTIONS_GENERAL: slotOptionsGeneral(); break;
   case ID_OPTIONS_KEY:     slotOptionsKey(); break;
   case ID_OPTIONS_FONT:     slotOptionsFont(); break;
   case ID_OPTIONS_COLORS: slotOptionsColors(); break;

   case ID_RIGHT_HIDDEN: slotRightHidden(); break;
   case ID_RIGHT_SORTNAME: slotRightSortName(); break;
   case ID_RIGHT_SORTSIZE: slotRightSortSize(); break;
   case ID_RIGHT_SORTATTRIB: slotRightSortAttrib(); break;
   case ID_RIGHT_SORTOWNER: slotRightSortOwner(); break;
   case ID_RIGHT_SORTGROUP: slotRightSortGroup(); break;
   case ID_RIGHT_SORTLASTMODTIME: slotRightSortLastModTime(); break;
   case ID_RIGHT_SORTLASTREADTIME: slotRightSortLastReadTime(); break;
    case ID_RIGHT_REFRESH: slotRightRefresh(); break;
   default:	break;
  }
  if ((id >= ID_BOOKMARK_MIN) && (id <= ID_BOOKMARK_MAX))
    slotBookmarkChosen(id);
}

void KNCApp::statusCallback(int id)
{
  QString StatusMsg;
  PluginFactoryIterator it = PluginAdmin::instance().getPluginFactoryIterator();
  for (;it.current();++it)
    if (it.current()->statusCallback(id, StatusMsg))
    {
      slotStatusHelpMsg(StatusMsg);
      return;
    }

  switch (id){
   case ID_LEFT_HIDDEN: 	slotStatusHelpMsg(i18n("Shows or hides hidden files in left box."));	break;
   case ID_LEFT_SORTNAME: slotStatusHelpMsg(i18n("Sorts left box by name.")); break;
   case ID_LEFT_SORTSIZE: slotStatusHelpMsg(i18n("Sorts left box by size.")); break;
   case ID_LEFT_SORTATTRIB: slotStatusHelpMsg(i18n("Sorts left box by attribute.")); break;
   case ID_LEFT_SORTOWNER: slotStatusHelpMsg(i18n("Sorts left box by owner.")); break;
   case ID_LEFT_SORTGROUP: slotStatusHelpMsg(i18n("Sorts left box by group.")); break;
   case ID_LEFT_SORTLASTMODTIME: slotStatusHelpMsg(i18n("Sorts left box by last modified time.")); break;
   case ID_LEFT_SORTLASTREADTIME: slotStatusHelpMsg(i18n("Sorts left box by last read time.")); break;
   case ID_LEFT_REFRESH: slotStatusHelpMsg(i18n("Refreshes left box.")); break;

   case ID_COMMAND_SELECT: slotStatusHelpMsg(i18n("Select files matching wildcard.")); break;
   case ID_COMMAND_UNSELECT: slotStatusHelpMsg(i18n("Unselect files matching wildcard.")); break;
   case ID_COMMAND_VIEW: slotStatusHelpMsg(i18n("Views selected files.")); break;
   case ID_COMMAND_EDIT: slotStatusHelpMsg(i18n("Edits selected files.")); break;
   case ID_COMMAND_COPY: slotStatusHelpMsg(i18n("Copies selected files.")); break;
   case ID_COMMAND_MOVE: slotStatusHelpMsg(i18n("Move selected files.")); break;
   case ID_COMMAND_NEWDIR: slotStatusHelpMsg(i18n("Creates a new directory in selected box.")); break;
   case ID_COMMAND_DELETE: slotStatusHelpMsg(i18n("Deletes selected files.")); break;
   case ID_COMMAND_CHANGEATTRIB: slotStatusHelpMsg(i18n("Changes the attributes of the selected files/dirs.")); break;
   case ID_COMMAND_OPENTERMINAL: slotStatusHelpMsg(i18n("Opens a Terminal")); break;
   case ID_COMMAND_OPENFINDUTILITY: slotStatusHelpMsg(i18n("Opens a find utility")); break;
   case ID_COMMAND_INVOKECOMMANDLINE: slotStatusHelpMsg(i18n("Switches to the command line.")); break;
   case ID_COMMAND_QUIT: slotStatusHelpMsg(i18n("Exits KNC.")); break;

   case ID_VIEW_TOOLBAR: slotStatusHelpMsg(i18n("Shows or hides the toolbar.")); break;
   case ID_VIEW_STATUSBAR: slotStatusHelpMsg(i18n("Shows or hides the statusbar.")); break;
   case ID_VIEW_DIRECTORYLINE: slotStatusHelpMsg(i18n("Shows or hides the directory line.")); break;
   case ID_VIEW_COMMANDLINE: slotStatusHelpMsg(i18n("Shows or hides the command line.")); break;

   case ID_BOOKMARK_ADD: slotStatusHelpMsg(i18n("Adds the current side as bookmark.")); break;
   case ID_BOOKMARK_REMOVE: slotStatusHelpMsg(i18n("Removes a bookmark.")); break;

   case ID_OPTIONS_GENERAL: slotStatusHelpMsg(i18n("Shows general options dialog.")); break;
   case ID_OPTIONS_KEY: slotStatusHelpMsg(i18n("Configure the key accelerators.")); break;
   case ID_OPTIONS_FONT: slotStatusHelpMsg(i18n("Shows dialog for setting the box-font.")); break;
   case ID_OPTIONS_COLORS: slotStatusHelpMsg(i18n("Shows a dialog for setting different colors.")); break;

   case ID_RIGHT_HIDDEN: slotStatusHelpMsg(i18n("Shows or hides hidden files in right box.")); break;
   case ID_RIGHT_SORTNAME: slotStatusHelpMsg(i18n("Sorts right box by name.")); break;
   case ID_RIGHT_SORTSIZE: slotStatusHelpMsg(i18n("Sorts right box by size.")); break;
   case ID_RIGHT_SORTATTRIB: slotStatusHelpMsg(i18n("Sorts right box by attribute.")); break;
   case ID_RIGHT_SORTOWNER: slotStatusHelpMsg(i18n("Sorts right box by owner.")); break;
   case ID_RIGHT_SORTGROUP: slotStatusHelpMsg(i18n("Sorts right box by group.")); break;
   case ID_RIGHT_SORTLASTREADTIME: slotStatusHelpMsg(i18n("Sort right box by last read time.")); break;
   case ID_RIGHT_SORTLASTMODTIME: slotStatusHelpMsg(i18n("Sort right box by last modified time.")); break;
   case ID_RIGHT_REFRESH: slotStatusHelpMsg(i18n("Refreshes right box.")); break;
 	 default:
			break;
  }

}

void KNCApp::setActListBox (bool LeftBox)
{
  if ((LeftBox) && (!_Act))
  {
    _Act = true;
    _ActListBox = _LeftBox;
    _ActListBox->setFocus();
    emit currentDirChanged(_ActListBox->getDir());
  }
  if ((!LeftBox) && (_Act))
  {
    _Act = false;
    _ActListBox = _RightBox;
    _ActListBox->setFocus();
    emit currentDirChanged(_ActListBox->getDir());
  }
}


void KNCApp::initView()
{
  _CommandPanner = new KNewPanner(this, 0, KNewPanner::Horizontal); //, KNewPanner::Absolute);
  _CommandWidget = new MyCommandWidget(_CommandPanner);

  _OuterPanner = new KNewPanner(_CommandPanner, 0, KNewPanner::Horizontal, KNewPanner::Percent, 0);
  _ProtocolView = new QMultiLineEdit(_OuterPanner);
  initPlugins();

  _Panner = new KNewPanner(_CommandPanner);

  _LeftBox = new MyListView(_Panner, 0, _ProtocolView);
  _RightBox = new MyListView(_Panner, 0, _ProtocolView);
  _ActListBox = _LeftBox;

  _CommandPanner->activate( _OuterPanner, _CommandWidget);
  _OuterPanner->activate(_ProtocolView, _Panner);
  _Panner->activate(_LeftBox, _RightBox);

  _Panner->setLabels("/", "/");
  _Panner->showLabels(true);

  _ProtocolView->setReadOnly(true);

  _config->setGroup("Font");
  QFont Font;
  _config->setGroup("General Options");
  Font = _config->readFontEntry("Font", &Font);
  _LeftBox->setFont(Font);
  _RightBox->setFont(Font);
  _config->setGroup("Column Width");
  _LeftBox->setColumnWidth(0,_config->readNumEntry("lName", 190));
  _LeftBox->setColumnWidth(1,_config->readNumEntry("lSize", 90));
  _LeftBox->setColumnWidth(2,_config->readNumEntry("lAttrib", 100));
  _LeftBox->setColumnWidth(3,_config->readNumEntry("lOwner", 90));
  _LeftBox->setColumnWidth(4,_config->readNumEntry("lGroup", 90));
  _LeftBox->setColumnWidth(5,_config->readNumEntry("lLastMod", 220));
  _LeftBox->setColumnWidth(6,_config->readNumEntry("lLastRead", 220));
  _RightBox->setColumnWidth(0, _config->readNumEntry("rName", 190));
  _RightBox->setColumnWidth(1, _config->readNumEntry("rSize", 90));
  _RightBox->setColumnWidth(2, _config->readNumEntry("rAttrib", 100));
  _RightBox->setColumnWidth(3, _config->readNumEntry("rOwner", 90));
  _RightBox->setColumnWidth(4, _config->readNumEntry("rGroup", 90));
  _RightBox->setColumnWidth(5, _config->readNumEntry("rLastMod", 220));
  _RightBox->setColumnWidth(6, _config->readNumEntry("rLastRead", 220));

  setView(_CommandPanner);

  connect(_LeftBox, SIGNAL(focusIn()),SLOT(slotLeftBoxFocusIn()));
  connect(_RightBox, SIGNAL(focusIn()),SLOT(slotRightBoxFocusIn()));
  connect(_RightBox, SIGNAL(focusIn()),SLOT(slotRightBoxFocusIn()));
  connect(_CommandWidget->getCommandLine(), SIGNAL(focusIn()),SLOT(slotCommandLineFocusIn()));
  connect(_CommandWidget->getCommandLine(), SIGNAL(focusOut()),SLOT(slotCommandLineFocusOut()));
  connect(this, SIGNAL(currentDirChanged(QString)), _CommandWidget, SLOT(setWorkingDirectory(QString)));
  connect(_LeftBox, SIGNAL(directoryChanged(QString)), SLOT(slotDirectoryChanged(QString)));
  connect(_RightBox, SIGNAL(directoryChanged(QString)), SLOT(slotDirectoryChanged(QString)));
  connect(_LeftBox, SIGNAL(executeFile(QString)), SLOT(slotExecuteFile(QString)));
  connect(_RightBox, SIGNAL(executeFile(QString)), SLOT(slotExecuteFile(QString)));
  connect(_LeftBox, SIGNAL(cursorChanged()), SLOT(slotCursorChanged()));
  connect(_RightBox, SIGNAL(cursorChanged()), SLOT(slotCursorChanged()));
  connect(_LeftBox, SIGNAL(selectChanged()), SLOT(slotSelectionChanged()));
  connect(_RightBox, SIGNAL(selectChanged()), SLOT(slotSelectionChanged()));


  _LeftBox->setFocus();
}

void KNCApp::slotDirectoryChanged (QString Dir = "")
{
 _Panner->setLabels(_LeftBox->getDir(),_RightBox->getDir());
 if (Dir != "")
   emit currentDirChanged(Dir);
 else
   emit currentDirChanged(_ActListBox->getDir());
}

void KNCApp::slotChangeWindows ()
{
  if (_Act)
   setActListBox (false);
  else
   setActListBox (true);
}


const char* MyListViewItem::key(int column, bool ascending) const
{
  static QString MKey;
  if (column == 0)
  {
     MKey = _FileInfo->fileName();
     if (_FileInfo->isDir())
     {
        MKey = MKey.upper();
        MKey.prepend("!");
     }
     else MKey = MKey.lower();
  }
  if (column == 1) //size ausgewaehlt
  {
    QString SLength;
    if (_FileInfo->isDir()) SLength = "";
    else SLength.sprintf ("%i", _FileInfo->size());
    MKey.sprintf("%i %i", SLength.length(), _FileInfo->size());
  }
  if (column == 2) return text(2); //attrib
  if (column == 3) return text(3); //owner
  if (column == 4) return text(4); //group
  if ((column == 5) || (column == 6))
  {
    QDateTime DT;
    if (column == 5)  DT = _FileInfo->lastModified();
    if (column == 6)  DT = _FileInfo->lastRead();
    QString Month, Day, Hour, Minute, Second;
    Month.sprintf("%i",DT.date().month());
    Day.sprintf("%i", DT.date().day());
    Hour.sprintf("%i", DT.time().hour());
    Minute.sprintf("%i", DT.time().minute());
    Second.sprintf("%i", DT.time().second());
    MKey.sprintf ("%i %i %s %i %s %i %s %i %s %i %s", DT.date().year(), Month.length(), Month.data(), Day.length(), Day.data(),
                                      Hour.length(), Hour.data(), Minute.length(), Minute.data(), Second.length(), Second.data());
  }
  return MKey;
}

void KNCApp::slotLeftBoxFocusIn()
{
 setActListBox(true);
}
void KNCApp::slotRightBoxFocusIn()
{
 setActListBox(false);
}

void KNCApp::enableMessages(bool bEnable)
{
  key_accel->setItemEnabled ("Change Windows", bEnable);
  key_accel->setItemEnabled ("Directory Up", bEnable);
  key_accel->setItemEnabled ("Up", bEnable);
  key_accel->setItemEnabled ("Down", bEnable);
  key_accel->setItemEnabled ("Left", bEnable);
  key_accel->setItemEnabled ("Right", bEnable);
  key_accel->setItemEnabled ("Page Up", bEnable);
  key_accel->setItemEnabled ("Page Down", bEnable);
  key_accel->setItemEnabled ("Scroll Begin", bEnable);
  key_accel->setItemEnabled ("Scroll End", bEnable);
  key_accel->setItemEnabled ("Invert Selection", bEnable);
  key_accel->setItemEnabled ("Enter", bEnable);
  key_accel->setItemEnabled ("Execute with", bEnable);
  key_accel->setItemEnabled ("Reset Substring", bEnable);
  key_accel->setItemEnabled ("Refresh Left", bEnable);
  key_accel->setItemEnabled ("Refresh Right", bEnable);
  key_accel->setItemEnabled ("Selection", bEnable);
  key_accel->setItemEnabled ("Unselection", bEnable);
  key_accel->setItemEnabled ("View", bEnable);
  key_accel->setItemEnabled ("Edit", bEnable);
  key_accel->setItemEnabled ("Copy", bEnable);
  key_accel->setItemEnabled ("Move", bEnable);
  key_accel->setItemEnabled ("NewDir", bEnable);
  key_accel->setItemEnabled ("Delete", bEnable);
  key_accel->setItemEnabled ("Open Terminal", bEnable);
  key_accel->setItemEnabled ("Open Find Utility", bEnable);
  key_accel->setItemEnabled ("Change Attributes", bEnable);
}

void KNCApp::slotCommandLineFocusIn()
{
 enableMessages(false);
}

void KNCApp::slotCommandLineFocusOut()
{
 enableMessages(true);
}

void KNCApp::slotOptionsKey()
{
  if( KKeyDialog::configureKeys( key_accel ) )
  {
    changeMenuAccels();
  }
}

void formatString(QString& Number)
{
  int i=Number.length();
  int count = 3;
  while (i>0)
  {
    i-=count;
    if (i>0) Number.insert(i, ".");
  }
}

void KNCApp::slotSelectionChanged()
{
  if (!_ActListBox->isSelection()) return;

  QString Information, Files, Space;
  Files.setNum(_ActListBox->getSelectionFileCount(),'g',25);
  Space.setNum(_ActListBox->getSelectionFileSize(),'g',25);
  formatString(Files);
  formatString(Space);
  Information = Files + i18n(" Files selected using ") + Space + i18n(" bytes space");

  if (_ActListBox == _LeftBox)
    ((MyStatusBar*)statusBar())->setLeftSide(Information);
  else
    ((MyStatusBar*)statusBar())->setRightSide(Information);

}
void KNCApp::slotCursorChanged()
{
  QString Information, Help;
  if (_ActListBox->isSelection()) return;
  else
  {
    MyListViewItem* Item = (MyListViewItem*) _ActListBox->currentItem();
    if (Item) Information = Item->getFileInfo()->infoAsString ();
  }

  if (_ActListBox == _LeftBox)
    ((MyStatusBar*)statusBar())->setLeftSide(Information);
  else
    ((MyStatusBar*)statusBar())->setRightSide(Information);

}

void KNCApp::slotBookmarkAdd()
{
  if (_nBookmarks > MAX_BOOKMARKS) return;

  int id = _nBookmarks;
  QString Dir = _ActListBox->getDir();
  _bookmarks[id] = Dir;
  _bookmark_menu->insertItem(Dir, id+ID_BOOKMARK_MIN);
  _nBookmarks++;
}

void KNCApp::slotBookmarkChosen(int id)
{
 id = id - ID_BOOKMARK_MIN;
 _ActListBox->setDirectory (_bookmarks[id]);
}

void KNCApp::slotBookmarkRemove()
{
  QStrList BookmarkData;
  for (int i=0;i<_nBookmarks;i++) if (_bookmarks[i] != "") BookmarkData.append(_bookmarks[i]);

  ListBoxDialog Dialog(BookmarkData, i18n("Remove Bookmark"), i18n("Remove"));
  if (Dialog.exec())
  {
     if (Dialog.getType() != -1)
     {
       _bookmark_menu->removeItem(ID_BOOKMARK_MIN+Dialog.getType());
       _bookmarks[Dialog.getType()] = "";
     }
  }
}