/***************************************************************************
                          mycommandline.cpp  -  description
                             -------------------
    begin                : Sun Jan 30 2000
    copyright            : (C) 2000 by Henrik Stormer
    email                : stormer@ifi.unizh.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "mycommandline.h"
#include <qmessagebox.h>
#include <kapp.h>
#include <qkeycode.h>
#include "twolistboxesdialog.h"

MyCommandLine::MyCommandLine(QWidget *parent, const char *name ) : KLined(parent,name)
{
}
MyCommandLine::~MyCommandLine()
{
}

void MyCommandLine::keyPressEvent(QKeyEvent* e)
{
  if ( (e->state() == 0) && (e->key() == Key_Up || e->key() == Key_Down))
  {
    if (e->key() == Key_Up)
      emit rollUp();
    else
      emit rollDown();
	  cursorAtEnd();
	  return;
  }
  KLined::keyPressEvent(e);
}


MyCommandWidget::MyCommandWidget(QWidget *parent, const char *name )
  : QWidget(parent,name),_CommandLine(this), _DirLabel(this), _Parent(parent), _Completion(), _HistoryIt(_History)
{
 connect (&_Completion, SIGNAL (setText(const char*)), &_CommandLine, SLOT(setText(const char*)));
 connect (&_CommandLine, SIGNAL (rollUp()), SLOT(slotRollUp()));
 connect (&_CommandLine, SIGNAL (rollDown()), SLOT(slotRollDown()));
 connect (&_CommandLine, SIGNAL (textChanged(const char*)), SLOT(slotTextChanged(const char*)));

 setMinimumHeight(_CommandLine.height());
 connect( &_CommandLine, SIGNAL(returnPressed()), SLOT(slotExecute()));

 connect( &_CommandLine, SIGNAL(completion()), SLOT(slotCompletion()));
 connect( &_CommandLine, SIGNAL(rotation()), SLOT(slotRotation()));

 _History.setAutoDelete(true);
}

void MyCommandWidget::repaintCommandLine()
{
 int startLine = _DirLabel.width() + 10;
 _CommandLine.move(startLine, 0);
 _CommandLine.resize(_Parent->width() - startLine,_CommandLine.height());
}

void MyCommandWidget::slotRollUp()
{
 if (_HistoryIt.current())
 {
   _CommandLine.setText(_HistoryIt.current()->data());
   if (!_HistoryIt.atFirst()) --_HistoryIt;
 }
}

void MyCommandWidget::slotRollDown()
{
 if (_HistoryIt.current())
 {
   _CommandLine.setText(_HistoryIt.current()->data());
   if (!_HistoryIt.atLast()) ++_HistoryIt;
 }
}

void MyCommandWidget::slotTextChanged(const char*)
{

}

void MyCommandWidget::resizeEvent(QResizeEvent*)
{
 repaintCommandLine();
}

void MyCommandWidget::setWorkingDirectory(QString WorkingDir)
{
 _WorkingDir = WorkingDir;
 _DirLabel.move(5,0);
 QFont f(_DirLabel.font());
 QFontMetrics fm (f);
 _DirLabel.resize(fm.width(_WorkingDir+">"),_DirLabel.height());
 _DirLabel.setText(_WorkingDir+">");
 repaintCommandLine();
}

void MyCommandWidget::slotCompletion()
{
 _Completion.edited(_CommandLine.text());
 _Completion.make_completion(_WorkingDir);
}

void MyCommandWidget::slotRotation()
{
 _Completion.make_rotation();
}

void MyCommandWidget::slotReceivedStdout (KProcess* P, char* buffer, int length)
{
  QString Msg (buffer, length+1);
  if (Msg != "")
  {
    if (!_ProcessDictStdOut[(long)P])
      _ProcessDictStdOut.insert((long)P, new QString (Msg));
    else
      _ProcessDictStdOut[(long)P]->append(Msg);
  }
}

void MyCommandWidget::slotReceivedStdErr (KProcess* P, char* buffer, int length)
{
  QString Msg (buffer, length+1);
  if (Msg != "")
  {
    if (!_ProcessDictStdErr[(long)P])
      _ProcessDictStdErr.insert((long)P, new QString (Msg));
    else
      _ProcessDictStdErr[(long)P]->append(Msg);
  }
}

void MyCommandWidget::slotProcessExited(KProcess* P)
{
  bool show = false;
  TwoListboxesDialog Dialog(i18n("Output:"), i18n("Errors:"));
  if (_ProcessDictStdOut[(long)P])
  {
    show = true;
    Dialog.insertIntoFirstListbox(*_ProcessDictStdOut[(long)P]);
    _ProcessDictStdOut.remove((long)P);
  }
  if (_ProcessDictStdErr[(long)P])
  {
    show = true;
    Dialog.insertIntoSecondListbox(*_ProcessDictStdErr[(long)P]);
    _ProcessDictStdErr.remove((long)P);
  }
  if (show) Dialog.exec();
}

void MyCommandWidget::slotExecute()
{
  KShellProcess* P = new KShellProcess();

  connect (P, SIGNAL(receivedStderr (KProcess*, char*, int)),
           this, SLOT (slotReceivedStdErr (KProcess*, char*, int)));

  connect (P, SIGNAL(receivedStdout (KProcess*, char*, int)),
           this, SLOT (slotReceivedStdout (KProcess*, char*, int)));

  connect (P, SIGNAL(processExited(KProcess*)),
           this, SLOT (slotProcessExited (KProcess*)));

  P->clearArguments();
  (*P) << "cd " << _WorkingDir << "&&";
  (*P) << _CommandLine.text();
  P->start(KProcess::NotifyOnExit, KProcess::AllOutput);

  _History.append(new QString(_CommandLine.text()));
  _HistoryIt.toLast();
  _CommandLine.setText("");
}