/***************************************************************************
                          mylistview.cpp  -  description                              
                             -------------------                                         
    begin                : Mon Aug 9 1999                                           
    copyright            : (C) 1999 by Henrik Stormer                         
    email                : stormer@ifi.unizh.ch                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <iostream.h>
#include "qapplication.h"
#include "qstatusbar.h"
#include "mylistview.h"
#include <kapp.h>
#include <kiconloader.h>
#include <sys/types.h>
#include <dirent.h>

#include "knc.h"
#include "generalfileinfo.h"
#include "generalfileplugin.h"
#include "tarfileplugin.h"
#include "ftpplugin.h"
#include "pluginadmin.h"
#include "splashscreen.h"
#include "qdict.h"

MyListView::MyListView ( QWidget * parent, const char * name, QMultiLineEdit* ProtocolView ) :
  QListView ( parent, name ), _bShowHiddenFiles(true), _bSelectionDirty(false), _ActivePlugin(NULL)
{
// it is impossible to reconnect the header signal. So sorting is only done ascending
  setFocusPolicy(QWidget::StrongFocus);

  connect(this, SIGNAL(doubleClicked(QListViewItem*)), SLOT(slotEnter()));

  _ncName = addColumn(i18n("Name"), 190);
  _ncSize = addColumn(i18n("Size"), 90);
  _ncAttrib = addColumn(i18n("Attribute"), 100);
  _ncOwner = addColumn(i18n("Owner"), 90);
  _ncGroup = addColumn(i18n("Group"), 90);
  _nclMod = addColumn(i18n("Last Modified"), 220);
  _nclRead = addColumn(i18n("Last Read"), 220);

  setMultiSelection (true);
  setAllColumnsShowFocus(true);
  _Dir = "/";
  _RecurseSubdir.setAutoDelete(true);
}

MyListView::~MyListView()
{}

MyListViewItem* MyListView::findItem(BaseFileInfo* f)
{
  MyListViewItem* Item = (MyListViewItem*)firstChild();
  while (Item)
  {
    if (Item->getFileInfo() == f) return Item;
    Item = (MyListViewItem*)Item->itemBelow();
  }
  return NULL;
}

MyListViewItem* MyListView::findItem(QString Name)
{
 MyListViewItem* Item = (MyListViewItem*)firstChild();
 while (Item != NULL)
 {
   if (Item->getFileInfo()->fileName() == Name) break;
   Item = (MyListViewItem*)Item->itemBelow();
 }

 return Item;
}

void MyListView::removeItem(MyListViewItem* Item)
{
  if (_SelectedList.find(Item) != -1) _SelectedList.remove();
  if (Item != NULL) delete Item;
}

MyListViewItem* MyListView::insertItem(BaseFileInfo* f)
{
  MyListViewItem* Item = new MyListViewItem(this, f, f->fileName(), f->getSize(), f->getAttrib(),
                                            f->getOwner(), f->getGroup(), f->lastModified().toString(), f->lastRead().toString());
  if (f->fileName() == "..") Item->setSelectable(false);
  return Item;
}

void MyListView::traverseDirectory(QString Dir, QString CurrentItemName)
{
  _SelectedList.clear();
  QString OldDir = _Dir;
  _Dir = Dir;
  BasePlugin* Plugin = PluginAdmin::instance().getResponsiblePlugin(_Dir,"");
  if ((!Plugin) || (!Plugin->create(Dir, this, CurrentItemName, _bShowHiddenFiles)))
  {
   //something went wrong, first try to use the old Plugin
    bool success = false;
    if (_ActivePlugin)
    {
      //swap Plugin and _ActivePlugin for deleting Plugin if _ActivePlugin succeeds
      BasePlugin* TempPlugin = Plugin;
      Plugin = _ActivePlugin;
      _ActivePlugin = TempPlugin;

      success = Plugin->create(_Dir = OldDir, this, "", _bShowHiddenFiles);
    }
    //if the old plugin also failed, we try the root dir at last
    if (!success)
    {
      Plugin = PluginAdmin::instance().getResponsiblePlugin(_Dir="/","");
      Plugin->create(_Dir, this, CurrentItemName, _bShowHiddenFiles);
    }
  }
  //now compare Plugin and ActivePlugin
  if (_ActivePlugin != Plugin)
  {
    //Plugin canrenot be null, ActivePlugin can!
    if (_ActivePlugin)
      _ActivePlugin->getsInactive(this);
    Plugin->getsActive(this);
    _ActivePlugin = Plugin;
  }


  slotResetSubString();
  qApp->processEvents();
  emit directoryChanged(_Dir);
  emit cursorChanged();
  emit selectChanged();
}

void MyListView::traverseDirectoryDown(BaseFileInfo* Info)
{
  if (!Info->accessExecute()) return;
  _SelectedList.clear();
  QString OldDir = _Dir;

  //look for the new Plugin
  BasePlugin* NewPlugin = PluginAdmin::instance().getResponsiblePlugin(_Dir, Info->fileName());
  //and get the new directory
  if (NewPlugin) _Dir = NewPlugin->cdDown(_Dir, Info->fileName());

  //now the same as directory up
  if ((!NewPlugin)||(!NewPlugin->create(_Dir, this, "", _bShowHiddenFiles)))
  {
    bool success = false;
    if (_ActivePlugin)
    {
      BasePlugin* TempPlugin = NewPlugin;
      NewPlugin = _ActivePlugin;
      _ActivePlugin = TempPlugin;

      success = NewPlugin->create(_Dir = OldDir, this, "", _bShowHiddenFiles);
    }
    //if the old plugin also failed, we try the root dir at last
    if (!success)
    {
      NewPlugin = PluginAdmin::instance().getResponsiblePlugin(_Dir="/","");
      NewPlugin->create(_Dir, this, "", _bShowHiddenFiles);
    }
  }

  if (_ActivePlugin != NewPlugin)
  {
    //NewPlugin cannot be null, ActivePlugin can!
    if (_ActivePlugin)
      _ActivePlugin->getsInactive(this);
    NewPlugin->getsActive(this);
    _ActivePlugin = NewPlugin;
  }

  slotResetSubString();
  qApp->processEvents();
  emit directoryChanged(_Dir);
  emit cursorChanged();
  emit selectChanged();
}

//this Method is called when the user wants to traverse a directory up
void MyListView::traverseDirectoryUp()
{
  QString Name;
  _SelectedList.clear();
  QString OldDir = _Dir;
  //first, get the new directory from the active plugin
  if (_ActivePlugin) _Dir = _ActivePlugin->cdUp(_Dir, Name);

  //and look for the new plugin
  BasePlugin* NewPlugin = PluginAdmin::instance().getResponsiblePlugin(_Dir,"");

  if ((!NewPlugin)||(!NewPlugin->create(_Dir, this, Name, _bShowHiddenFiles)))
  {
    bool success = false;
    if (_ActivePlugin)
    {
      BasePlugin* TempPlugin = NewPlugin;
      NewPlugin = _ActivePlugin;
      _ActivePlugin = TempPlugin;

      success = NewPlugin->create(_Dir = OldDir, this, Name, _bShowHiddenFiles);
    }
    //if the old plugin also failed, we try the root dir at last
    if (!success)
    {
      NewPlugin = PluginAdmin::instance().getResponsiblePlugin(_Dir="/","");
      NewPlugin->create(_Dir, this, "", _bShowHiddenFiles);
    }
  }

  if (_ActivePlugin != NewPlugin)
  {
    //NewPlugin cannot be null, ActivePlugin can!
    if (_ActivePlugin)
      _ActivePlugin->getsInactive(this);
    NewPlugin->getsActive(this);
    _ActivePlugin = NewPlugin;
  }

  slotResetSubString();
  qApp->processEvents();
  emit directoryChanged(_Dir);
  emit cursorChanged();
  emit selectChanged();
}

void MyListView::slotEnter()
{
  if (!currentItem()) return;
  MyListViewItem* Item = (MyListViewItem*) currentItem();
  BasePlugin* Plugin;
  if (Item->getFileInfo()->isDir())
  {
     if (Item->getFileInfo()->fileName() != "..") traverseDirectoryDown(Item->getFileInfo());
     else traverseDirectoryUp();
  }
  else if (Item->getFileInfo()->isFile())
  {
    BasePlugin* OldPlugin = PluginAdmin::instance().getResponsiblePluginInstance(Item->getFileInfo()->dirPath(),"");
    Plugin = PluginAdmin::instance().getResponsiblePlugin(Item->getFileInfo()->dirPath(),Item->getFileInfo()->fileName());
    if ((Plugin != NULL) && (Plugin != OldPlugin))
      traverseDirectoryDown(Item->getFileInfo());

    else emit executeFile(Item->getFileInfo()->getFile());
  }
}

void MyListView::makeItemVisible(MyListViewItem* Item)
{
 if (Item)
 {
   int dy =  contentsY() - Item->itemPos();
   if (dy > 0) scrollBy(0, -dy);

   dy = (Item->itemPos() + Item->height()) - (contentsY() + viewport()->height());
   if (dy > 0) scrollBy(0, dy);
 }
}

void MyListView::slotUp ()
{
 if (currentItem())
 {
   MyListViewItem* Item = (MyListViewItem*)currentItem();

   if (_SubString != NULL) Item = findSubStringItem(false, _SubString);
   else Item = (MyListViewItem*)Item->itemAbove();

   if (Item) setCurrentItem(Item);
   else Item = (MyListViewItem*)currentItem();
   makeItemVisible(Item);
   emit cursorChanged();
 }
}

void MyListView::slotDown()
{
 if (currentItem())
 {
   MyListViewItem* Item = (MyListViewItem*)currentItem();

   if (_SubString != NULL) Item = findSubStringItem(true, _SubString);
   else Item = (MyListViewItem*)Item->itemBelow();

   if (Item) setCurrentItem(Item);
   else Item = (MyListViewItem*)currentItem();
   makeItemVisible(Item);
   emit cursorChanged();
 }
}

void MyListView::slotLeft()
{
 if (currentItem())
 {
   MyListViewItem* Item = (MyListViewItem*)currentItem();
   if (_SubString != NULL)
   {
     if (_SubString != "")
     {
       _SubString.truncate(_SubString.length()-1);
       Item->repaint();
     }
   }
   else scrollBy(-10,0);
 }
}

void MyListView::slotRight()
{
if (currentItem())
 {
   MyListViewItem* Item = (MyListViewItem*)currentItem();
   if (_SubString != NULL)
   {
     if (Item->getFileInfo()->fileName().length() > _SubString.length())
     {
       _SubString += Item->getFileInfo()->fileName()[_SubString.length()];
       Item->repaint();
     }
   }
   else scrollBy(10,0);

 }
}

void MyListView::slotPageUp()
{
 if (currentItem())
 {
   MyListViewItem* Item = (MyListViewItem*) currentItem();
   int count = viewport()->height() / Item->height();
   while ((count-- > 0) && (Item->itemAbove() != NULL))
     Item = (MyListViewItem*)Item->itemAbove();

   if (_SubString != NULL)
     if (!matchItem(Item, _SubString)) return;

   setCurrentItem(Item);
   makeItemVisible(Item);
   emit cursorChanged();
 }
}

void MyListView::slotPageDown()
{
 if (currentItem())
 {
   MyListViewItem* Item = (MyListViewItem*) currentItem();
   int count = viewport()->height() / Item->height();
   while ((count-- > 0) && (Item->itemBelow() != NULL))
     Item = (MyListViewItem*)Item->itemBelow();

   if (_SubString != NULL)
     if (!matchItem(Item, _SubString)) return;

   setCurrentItem(Item);
   makeItemVisible(Item);
   emit cursorChanged();
  }
}

void MyListView::slotScrollBegin()
{
 if (currentItem())
 {
   MyListViewItem* Item = (MyListViewItem*)firstChild();
   slotResetSubString();
   setCurrentItem(Item);
   makeItemVisible(Item);
   emit cursorChanged();
 }
}

//es gibt leider keine lastChild Methode!
void MyListView::slotScrollEnd()
{
 if (currentItem())
 {
   MyListViewItem* Item = (MyListViewItem*) currentItem();
   while (Item->itemBelow() != NULL) Item = (MyListViewItem*)Item->itemBelow();

   slotResetSubString();
   setCurrentItem(Item);
   makeItemVisible(Item);
   emit cursorChanged();
 }
}

void MyListView::slotInvertSelection()
{
 if (currentItem())
 {
   MyListViewItem* Item = (MyListViewItem*) currentItem();
   if (Item->getFileInfo()->fileName() != "..")
   {
     setSelected(Item, !Item->isSelected());
     Item->repaint();
   }
   slotDown();
   emit selectChanged();
 }
}

void MyListView::slotDirectoryUp()
{
  traverseDirectoryUp();
}

bool MyListView::select(QRegExp RegExp, bool Selection)
{
  if (!RegExp.isValid()) return false;
  else
  {
    if (currentItem())
    {
      MyListViewItem* Item = (MyListViewItem*) firstChild();
      while (Item != NULL)
      {
        if (RegExp.match(Item->getFileInfo()->fileName()) != -1)
        {
          if (Item->getFileInfo()->fileName() != "..")
          {
            setSelected(Item, Selection);
            repaintItem(Item);
            emit selectChanged();
          }
        }
        Item = (MyListViewItem*)Item->itemBelow();
      }
    }
    return true;
  }
}


void MyListView::setSelected ( QListViewItem * Item, bool Selected )
{
  QListView::setSelected(Item, Selected);
  if (Selected) //insert Item in List
  {
     if (_SelectedList.find((MyListViewItem*)Item) == -1)
        _SelectedList.append((MyListViewItem*)Item);
  }
  else //remove item from List
  {
    if (_SelectedList.find((MyListViewItem*)Item) != -1) _SelectedList.remove();
  }
  _bSelectionDirty = true;
}


void MyListView::createSelectedList(QList<BaseFileInfo>& FileList, double& countFiles, double& sizeAll, bool bIncludeSubdirs, bool createList)
{
 sizeAll = countFiles = 0;
 bool added = false;
 MyListViewItem* addedItem;
 SplashScreen Splash(this,0,0,i18n("One moment, recursing subdirs..."));
 //check if something is selected
 if (_SelectedList.isEmpty())
 {
   if (!createList) return;
   added = true;
   addedItem = (MyListViewItem*) currentItem();
   _SelectedList.append(addedItem);
 }

 //search for responsible plugin
 BasePlugin* RespPlugin = PluginAdmin::instance().getResponsiblePlugin(_Dir,"");

 QListIterator<MyListViewItem> it(_SelectedList);
 while (it.current())
 {
   MyListViewItem* Item = (MyListViewItem*) it.current();
   if ((Item->getFileInfo()->isDir())&&(Item->getFileInfo()->fileName() != "..")&&(Item->getFileInfo()->fileName() != "."))
   {
      double recSize = 0;
      double recCount = 0;

      if (createList) FileList.append(Item->getFileInfo()->clone());

      if ((!createList) && (_RecurseSubdir[Item->getFileInfo()->filePath()]))
      {
         recSize = _RecurseSubdir[Item->getFileInfo()->filePath()]->size;
         recCount = _RecurseSubdir[Item->getFileInfo()->filePath()]->count;
      }
      else
      if (bIncludeSubdirs)
      {
        Splash.show();
        qApp->processEvents(2000);
        RespPlugin->recurseSubdir(Item->getFileInfo()->filePath() + "/", FileList, recCount, recSize);
        struct SC* s = new SC;
        s->size = recSize; s->count = recCount;
        _RecurseSubdir.insert(Item->getFileInfo()->filePath(), s);
      }
      countFiles += recCount; sizeAll += recSize;
   }
   else //all other things (links, symlinks, files, devices) are also put in the list
   {
      if (createList) FileList.append(Item->getFileInfo()->clone());
      sizeAll += Item->getFileInfo()->size();
      countFiles++;
   }
   ++it;
 }
 //if the current item was added -> remove it now because it is not selected
 if (added) _SelectedList.remove(addedItem);

 //if a List was created -> a file operation follows, so clear the subdir sizes
 if (createList) _RecurseSubdir.clear();
 Splash.close();
}

double MyListView::getSelectionFileSize()
{
 if (_bSelectionDirty)
 {
   QList<BaseFileInfo> dummy;
   createSelectedList( dummy, _SelectionFileCount, _SelectionFileSize, _bSelectionIncludeSubdirs ? true : false, false);
   _bSelectionDirty = false;
 }
 return _SelectionFileSize;
}

double MyListView::getSelectionFileCount()
{
 if (_bSelectionDirty)
 {
   QList<BaseFileInfo> dummy;
   createSelectedList( dummy, _SelectionFileCount, _SelectionFileSize, _bSelectionIncludeSubdirs ? true : false, false);
   _bSelectionDirty = false;
 }
 return _SelectionFileCount;
}


void MyListView::refresh(QString CurrentItemName)
{
  int x = contentsX();
  int y = contentsY();
  if ((currentItem()) && (CurrentItemName == ""))
  {
    MyListViewItem* Item = (MyListViewItem*)currentItem();
    CurrentItemName = Item->getFileInfo()->fileName();
  }                  		
  QString Path = _Dir;
//  _Dir = _Dir;
  traverseDirectory (_Dir, CurrentItemName);
  qApp->processEvents();

  if (_Dir != Path)
      emit directoryChanged(_Dir);
  else
    if (CurrentItemName != "") setContentsPos(x,y);

  if (currentItem()) ensureItemVisible(currentItem());
}

void MyListView::slotResetSubString()
{
  _SubString = NULL;
  currentItem()->repaint();
}

void MyListView::keyPressEvent ( QKeyEvent * e)
{
//  QListView::keyPressEvent(e);
  if ( !e ) return;
  if ( !currentItem() )	return;

  MyListViewItem* Item = (MyListViewItem*)currentItem();
  QString text = Item->text(0);
	if (e->ascii())
  {
     QString TSubString = _SubString + e->ascii();
     if (!matchItem(Item, TSubString))
        Item = findSubStringItem(true, TSubString);

     if (Item != NULL)
     {
        _SubString = TSubString;
        setCurrentItem( Item );
        makeItemVisible( Item );
        Item->repaint();
        emit cursorChanged();
     }
  }
}

bool MyListView :: matchItem(MyListViewItem* Item, QString SubString)
{
  QString Text = Item->text(0);
  Text = Text.left(SubString.length());
  if (Text == SubString) return true;
  else return false;
}

//finds the NEXT Item in up or down orientation
MyListViewItem* MyListView::findSubStringItem(bool down, QString SubString)
{
  if ( !currentItem() )	return NULL;
  MyListViewItem* Item = (MyListViewItem*)currentItem();
  MyListViewItem* StartItem = Item;
  while(1)
  {
    if (down) Item = (MyListViewItem*)Item->itemBelow(); else Item = (MyListViewItem*)Item->itemAbove();
    if (!Item)
    {
      if (down) Item = (MyListViewItem*)firstChild();
      else //Troll forgot to make a lastChild Method :-(
      {
        Item = StartItem;
        while(Item->itemBelow()) Item = (MyListViewItem*)Item->itemBelow();
      }
    }
    if (matchItem(Item, SubString)) return Item;

    if (Item == StartItem) return NULL;
  }
}

void MyListView :: mousePressEvent(QMouseEvent* e)
{
  if ( !e )
	return;
  MyListViewItem* Item = (MyListViewItem*) itemAt( e->pos() );
  if (!Item) return;

  if (e->button() == LeftButton)
  {
    if (_SubString != NULL) slotResetSubString();
    setCurrentItem(Item);
    emit cursorChanged();
  }
}

void MyListView :: mouseReleaseEvent(QMouseEvent* e)
{

}