/***************************************************************************
                          optionsdialog.cpp  -  description                              
                             -------------------                                         
    begin                : Mon Aug 9 1999                                           
    copyright            : (C) 1999 by Henrik Stormer                         
    email                : stormer@ifi.unizh.ch                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include "optionsdialog.h"
#include "kapp.h"
#include <pluginadmin.h>
#include <basepluginfactory.h>

OptionsDialog::OptionsDialog(QWidget *parent, const char *name, QString Viewer, QString Editor, QString Find, bool ShowDeleteMessageBox, bool ExecKfm, bool RecurseSubdirs)
  : QTabDialog(parent,name,true)

{
	initDialog();
  setCaption(name);
  setOkButton();
  setCancelButton();
  int width = 400;
  int height = 400;
  _TabList.setAutoDelete(true);

//  _ViewTab = new ViewTab(this);
  _GeneralTab = new GeneralTab(this, name, ShowDeleteMessageBox, ExecKfm, RecurseSubdirs);
  _ExternalTab = new ExternalTab(this, name, Viewer, Editor, Find);
//  addTab(_ViewTab, "View");
  addTab(_GeneralTab, (const char*) i18n("General"));
  addTab(_ExternalTab, (const char*) i18n("External Programs"));
  PluginFactoryIterator it = PluginAdmin::instance().getPluginFactoryIterator();
  for (;it.current();++it)
   if (BaseOptionTab* Tab = it.current()->getOptionsTab())
   {
     if (Tab->width() > width) width = Tab->width();
     if (Tab->height() > height) height = Tab->height();
     addTab(Tab,it.current()->getOptionsTabName());
     _TabList.append(Tab);
   }

  resize(width, height+80); //to include also titlebar and buttons
}

OptionsDialog::~OptionsDialog()
{
  delete _GeneralTab;
  delete _ExternalTab;
}

void OptionsDialog::done(int status)
{
  QListIterator<BaseOptionTab> it (_TabList);
  if (status == 1)
    for (;it.current();++it) it.current()->okPressed();
  else
    for (;it.current();++it) it.current()->cancelPressed();

  QDialog::done(status);
}


